c
c     This file contains a set of routines which are used for the
c     diagonalization and the direct energy minimization
c
c     ==================================================================
c
      subroutine overlap_matrix(ovlmat, lvec, rvec,m,len)

c
c     FLOPS: 8*m*m*len
c
      use all_to_all_module
      implicit none
      include 'all_to_all.h'
      include 'flibcalls.ph'

      complex(kind=8) ovlmat(*), lvec(*), rvec(*)
      integer m,len

      complex(kind=8) dcone, dczero,dcminustwo
      parameter(dcone=(1.d0,0.d0), dczero=(0.d0,0.d0),
     $     dcminustwo=(-2.d0,0.d0))

      call mzgemm ('C','N', m,m,len,  
     $     dcone, lvec(1), len, rvec(1),len, dczero, ovlmat(1), m)

      call fast_all_sum_all_alloc_dc(m*m)
      call fast_all_sum_all_complex(ovlmat(1),m*m)

      return
      end subroutine

c
c     ==================================================================
c
      subroutine cheap_overlap_matrix(ovlmat, vec,m,len)
c
c     uses zherk, and assumes lvec=rvec
c
c
c     FLOPS: 4*m*m*len
c
      use all_to_all_module
      implicit none
      include 'all_to_all.h'
      include 'flibcalls.ph'


      complex(kind=8) ovlmat(*), vec(*)
      integer m,len

      complex(kind=8) dcone, dczero,dcminustwo
      parameter(dcone=(1.d0,0.d0), dczero=(0.d0,0.d0),
     $     dcminustwo=(-2.d0,0.d0))

      call mzherk ('L','C', m,len,  
     $     dcone, vec(1), len, dczero, ovlmat(1), m)

      call fast_all_sum_all_alloc_dc(m*m)
      call fast_all_sum_all_complex(ovlmat(1),m*m)

      call matrix_complete(ovlmat,m) ! mirror to get all elements

      return
      end subroutine

c
c     ==================================================================
c
      subroutine hermit_overlap_matrix(ovlmat, lvec, rvec,m,len)
c
c     FLOPS: 4*m*m*len
c
      use all_to_all_module
      implicit none
      include 'all_to_all.h'
      include 'flibcalls.ph'

      
      complex(kind=8) dcone, dczero,dcminustwo
      parameter(dcone=(1.d0,0.d0), dczero=(0.d0,0.d0),
     $     dcminustwo=(-2.d0,0.d0))

      complex(kind=8) ovlmat(*), lvec(*), rvec(*)
      integer m,len

      call mzher2k('L','C', m, len, dcone, lvec(1), 
     $     len, rvec(1), len, dczero, ovlmat(1),m)

      call fast_all_sum_all_alloc_dc(m*m)
      call fast_all_sum_all_complex(ovlmat(1),m*m)

      return
      end subroutine

c
c     ==================================================================
c
      subroutine printmatrix(mat, msize,nsize)
      implicit none
      integer msize,nsize
      integer i,j
      complex(kind=8) mat(msize,nsize)

      do i=1,msize
c         write(9,'(100(''('',f9.6,'','',f9.6,'')''))') 
c     $        (mat(i,j),j=1,nsize)
         write(9,'(100(''('',f15.12,'','',f15.12,'')''))') 
     $        (mat(i,j),j=1,nsize)
      end do
      end subroutine
c
c     ==================================================================
c
      subroutine printdmatrix(mat, msize,nsize)
      implicit none
      integer msize,nsize
      integer i,j
      double precision mat(msize,nsize)

      do i=1,msize
         write(9,'(100f15.6)')
     $        (mat(i,j),j=1,nsize)
      end do
      end subroutine
c
c     ==================================================================
c
      subroutine matrix_complete(mat, size)
c
      implicit none
      
      integer size,i,j
      complex(kind=8) mat(size,size)



      do i=1,size
         do j=1,i-1
            mat(j,i)=conjg(mat(i,j))
         end do
      end do

      end subroutine

c
c     ==================================================================
c
      subroutine matrix_polish(s,mat,w1,w2,m)
c
      implicit none
      include 'flibcalls.ph'

c
c     removes second order error from matrices
c
c
c     INPUT
c

      complex(kind=8) s(*)       ! s-matrix see below
      integer m                 ! size of matrix

c
c     INPUT/OUTPUT:
c
c     mat = (9*mat -3*mat*s - 3*s*mat + s^H * mat *s)/4
c

      complex(kind=8) mat(*)

c
c     WORK:
c
      complex(kind=8) w1(*), w2(*)
c
c     ------ local variables ---------------------
c
      integer i
      complex(kind=8) dcone, dczero,dcmthree,dcnine
      parameter(dcone=(1.d0,0.d0), dczero=(0.d0,0.d0),
     $     dcmthree=(-3.d0,0.d0),dcnine=(9.d0,0.d0))

      call mzgemm('N','N', m,m,m,dcone,mat(1),m,S(1),m,dczero,
     $     w1(1),m) 
      call mzgemm('C','N', m,m,m,dcone,s(1),m,w1(1),m,dczero,w2(1),m)
      call mzgemm('N','N', m,m,m,dcmthree,mat(1),m,S(1),m,
     $     dcone,w2(1),m)
      call mzgemm('C','N', m,m,m,dcmthree,S(1),m,mat(1),m,
     $     dcone,w2(1),m)
      call mzaxpy(m*m, dcnine, mat(1),1,w2(1),1)
      do i=1,m*m
         mat(i)=w2(i)/4.d0
      end do

      end subroutine
c
c     ==================================================================
c
      subroutine wavefn_rearrange(rows,cols, svec,dvec,iord,mnew)
c
c     rearranges wave functions according to order array
c
      implicit none
      include 'flibcalls.ph'

      complex(kind=8) svec(*),dvec(*)
      integer rows,cols, iord(*), mnew
c
c     
c
      integer i
      
      do i=1,mnew
         call mzcopy(rows,svec(rows*(iord(i)-1)+1),
     $        1,dvec(rows*(i-1)+1),1)
      end do
      end subroutine

c
c     ================== compute average kinetic energy =================
c
      double precision function average_ekin(x, tkinav,ekin,m,len)
      use all_to_all_module
      implicit none
      include 'all_to_all.h'


c
c     INPUT:
c     -----

      integer 
     $     m,                   ! number of vectors to precondition
     $     len                  ! length of vectors

      complex(kind=8) 
     $     x(len,m)             ! wave functions used to compute tkinav

      double precision 
     $     ekin(len)            ! kinetic energy of the gvectors |k+G|



      double precision tkinav(m) ! work array
c
c     DESCRIPTION:
c     computes and returns average kinetic energy

c     ---------- local variables -----------------
c
      integer i,j
      double precision eprecond,xsq

c     calculate maximum average kinetic energy of all trial vectors 

      
      do i=1,m
         tkinav(i)=0
         do j=1,len
            xsq=dble(x(j,i)*conjg(x(j,i)))
            tkinav(i) = tkinav(i)+xsq*ekin(j)
         end do
      end do

      call all_sum_all(tkinav,m)
      average_ekin = maxval(tkinav)

      end function

c
c     ==================================================================
c
c     @process extchk
      subroutine precondition(v, eprecond, ekinmod,ekin,m,len)
      use all_to_all_module
      implicit none
      include 'all_to_all.h'

c
c     INPUT:
c     -----

      integer 
     $     m,                   ! number of vectors to precondition
     $     len                  ! length of vectors

      double precision 
     $     eprecond,            ! the preconditioning energy cutoff
     $     ekin(len),           ! kinetic energy of the gvectors |k+G|
     $     ekinmod(3)           ! kinetic energy modification parameters

c
c     INPUT/OUTPUT:
c     ------------

      complex(kind=8) 
     $     v(len,m)             ! vectors to be preconditioned

c
c     DESCRIPTION:
c     preconditions vectors v(len,m) with the preconditioner 
c     of Teter, Payne, and Allen
c

c     ---------- local variables -----------------
c
c
c
      integer i,j
      complex(kind=8) gt
      double precision  pcfn, precfn
      external precfn
      double precision myderf

      external myderf

      do i=1,m
         do j=1,len
            pcfn = precfn((ekin(j)+ekinmod(1)
     $           *(1.0+myderf((ekin(j)-ekinmod(1))
     $           /ekinmod(2))))/eprecond)
            v(j,i)=v(j,i)*pcfn
         end do
      end do
      return

      end subroutine
      
c
c     ==================================================================
c
      double precision function precfn(x)
      implicit none
      double precision x

c
c     teter payne allan preconditioning
c
c
c      double precision x1,x2,x3,x4
c      x2=x*x
c      x3=x2*x
c      x4=x3*x
c
c      x1=27.d0+18.d0*x+12.d0*x2+8.d0*x3
c      precfn=x1/(x1+16.0*x4)


c
c      preconditioning a la f. mauri
c
      precfn=1.d0/max(1.d0,x)

      return
      end
c
c     ==================================================================
c
      double precision function trace(mat,m)
      implicit none
      integer m

      complex(kind=8) mat(m,m)

      integer i
      trace=0.d0
      do i=1, m
         trace=trace+dble(mat(i,i))
      end do

      return
      end function 
c
c     ==================================================================
c
      double precision function contract(mat1,mat2,m)
      implicit none
      integer m
      complex(kind=8) mat1(m,m), mat2(m,m)

      
      integer i,j
      double precision sum, odsum

      sum=0.d0
      odsum=0.d0
      do i=1, m
         sum=sum+dble(mat1(i,i))*dble(mat2(i,i))
         do j=i+1,m
            odsum = odsum + dble(mat1(j,i)*conjg(mat2(j,i)))
         end do
      end do

      contract = sum+2.d0*odsum
      return
      end function

