C*
C*
      subroutine ewald_dipole(ioutput, crys, gs, ewald_tens)
c
c
      use all_to_all_module
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
      include 'all_to_all.h'



c     INPUT:
c     -----

      type (crystal) crys       ! the crystal structure parameters
      type (parallel_gspace) gs ! the gspace structure and everything in it


      integer 
     $     ioutput              ! print flag

c     OUTPUT:
c     ------

      double precision 
     $     ewald_tens(3,3,crys%mxdatm*crys%ntype) 

c
c     DESCRIPTION:
c     -----------
c
c     computes the ewald field on each atomd due to the presence of
c     a dipole array. At each atom, there is a dipole attached of
c     strength crys%dipole(ntype), and all of the dipoles are
c     oriented in the same, but arbitrary direction (this direction
c     is the first tensor index). The dipole array gives rise to a
c     field, which is in the direction given by the second tensor
c     index, i.e.: 
c
c     Field(i) = Sum_j ewald_tens(j,i) * dipole_orientation(j)
c     
c     In case of the magnetic field, the dipole orientation is along
c     the external applied magnetic field, and the individual dipole
c     strengths are given by crys%dipole(ntype) from an atomic
c     calculation. 
c     
c     The summation used here is essentially from Akio Honma,
c     "Science of Light", vol25, No. 2 p 41 (1976). It is equation (2
c     .37), but without the factor of 4pi/3 P up front, and
c     generalized to give the dipole field at an arbitrary dipole
c     site in the unit cell.
c
c     To get the physical field at the dipole site, one has to add:
c
c     4pi/3 * polarization  + macroscopic field    (electric field)
c
c     -8pi/3 * magnetization + macroscopic field   (magnetic field)
c                                                      
c     So in the sense of Jackson, we have computed E_near, without
c     the Lorentz field. 
c

c     ---------------- local variables ----------------------------------
       
      integer
     $     alp, bet,            ! atom basis indices
     $     imx, jmx, kmx, ieta,
     $     natot,               ! total number of atoms, no matter which kind
     $     i, j, k, l, m, ir(3), im2, jm2, km2, itot, idum

      double precision
     $     tol, t0,
     $     eta,                 ! corresponds to Kittels eta
     $     seta,                ! sqrt(eta)
     $     aconst, kfundivx2, kfunthird, pi4invcell, inv4eta, inv2pi,
     $     rmax, xsi, kfun, invg2, xseta, xseta2,
     $     pi, twopi, small,  third, etalist(6),
     $     x, x2, x5, invx5, ep, er, f, g, ralp(3),
     $     prefac, g_times_r_alp, gc(3),
     $     gimmetime, myderfc

      data etalist /0.02, 0.5, 1, 2, 5, 10/

      parameter (pi = 3.141592653589793d0, twopi = 2.0d0*pi, 
     $     third=1.d0/3.d0)
      parameter (small = 1.0d-12)

      external gimmetime, myderfc
      

      complex(kind=8) 
     $     sumalp, dum
      complex(kind=8), allocatable :: stfac(:), tens(:,:,:)


c     work arrays
c
      double precision, allocatable ::
     $     dipole(:),           ! size dipole(crys%ntype*crys%mxdatm)
     $     rc(:,:)              ! size rc(3,crys%ntype*crys%mxdatm)

c
c     variables for the gspace loop:
c
      integer igs,iord,irod,igv(4),igv3,ffth(4),fftn(4)
      double precision gv(3)



c     -------------------------------------------------------------------

      t0= gimmetime()

      allocate(rc(3,crys%ntype*crys%mxdatm)) ;  rc =0
      allocate(dipole(crys%ntype*crys%mxdatm)); dipole=0
c
c     compute various parameters
c
      tol = -log(small)
      eta = 2.d0*gs%gmax/(4.d0*tol)

      seta = sqrt(eta)
      xsi = seta
      

      aconst = 4.d0*seta**3.d0/sqrt(pi)

      rmax = sqrt(tol/eta)
      imx = int(rmax*sqrt(crys%bdot(1,1))/twopi) + 1
      jmx = int(rmax*sqrt(crys%bdot(2,2))/twopi) + 1
      kmx = int(rmax*sqrt(crys%bdot(3,3))/twopi) + 1

      pi4invcell = 4.d0*pi/crys%vcell
      inv4eta = 1.d0/(4.d0*eta) ! 1/4 eta for the gspace decay
      inv2pi =1.d0/twopi
c
c     ----------------- store data for atoms into new arrays -----------
c
      natot = 0
      do i=1,crys%ntype
         do j=1,crys%natom(i)
            natot = natot + 1
            rc(:,natot) = crys%rat(:,j,i)
            dipole(natot) = crys%dipole(i)
         end do
      end do

      allocate(stfac(natot))
      allocate(tens(3,3,natot))
c     ---------------------------------------------------------------------
c                          DO SUM IN GSPACE
c
c     loop over gspace 

      fftn(1:3)=gs%fftsize(1:3)
      fftn(4)=gs%fftsize(3)
      ffth(:)=fftn(:)/2


      tens=0
      igs = 0 
      do iord=1, gs%lorder    ! loop through x/y gspace 
         irod=gs%order(1,iord)
         igv(1)= irod/gs%fftsize(2)
         igv(2)= mod(irod,gs%fftsize(2))
         igv(3)= gs%order(2,iord)
         igv(4)= gs%order(3,iord)
         igv(:)= mod(igv(:)+ffth(:),fftn(:))-ffth(:) 
         gv(1:2)=dble(igv(1:2))
         do igv3=igv(3),igv(4)     ! loop over z axis
            gv(3)=dble(igv3)
            igs = igs + 1
            if(gs%ekin(igs).gt.0.d0) then 
c
c              compute exp(-G^2/(4 eta)) G/|G|^2
c
               prefac= exp(-gs%ekin(igs)*inv4eta)
c
c              precompute the "structure factors" stfac. Here,
c              rc are the relative coordinates MULTIPLIED BY TWO PI!
c

               do alp=1,natot
                  g_times_r_alp = gv(1)*rc(1,alp)+
     $                 gv(2)*rc(2,alp)+gv(3)*rc(3,alp) ! rc has fac of twopi
                  stfac(alp)=cmplx(cos(g_times_r_alp),
     $                 sin(g_times_r_alp),kind=8)
               end do
c
c              now run the inner double loop over the pairs of atoms
c
c              tens(external_field, induced_field,atom)

               sumalp=0
               do alp=1, natot
                  sumalp = sumalp+conjg(stfac(alp))*dipole(alp) 
               end do
               sumalp=sumalp*prefac
               gc = matmul(crys%bvec,gv)
               invg2 = 1.d0/gs%ekin(igs)

               do bet=1, natot
                  dum = sumalp*stfac(bet)
                  do i=1,3
                     tens(i,:,bet)=tens(i,:,bet)
     $                    -gc(i)*gc(:)*dum*invg2
                     tens(i,i,bet)=tens(i,i,bet)+third*dum
                  end do
               end do
            endif
         end do 
      end do
c
c     put in the factor of -4pi/cell_volume
c
      tens = pi4invcell*tens

c
c     --------------------- now the realspace sum ------------------------
c     
c     Notice that the lattice sum does not scale with the size of the
c     system, i.e. the number of R vectors summed up is constant.
c     What does scale is the innermost double loop over the atoms,
c     and hence the routine is of complexity N**2.

      rc(:,1:natot)=rc(:,1:natot)*inv2pi ! remove factor of two pi

      im2 = 2*imx+1
      jm2 = 2*jmx+1
      km2 = 2*kmx+1

      idum=0
      do itot=gs%myproc, im2*jm2-1, gs%nproc
         i = itot/jm2+1
         j = mod(itot,jm2)+1
         ir(1) = i-imx-1
         ir(2) = j-jmx-1

         do k=1,km2
            ir(3) = k-kmx-1
            if((ir(1).ne.0).or.(ir(2).ne.0).or.(ir(3).ne.0))
     $           then
               idum=idum+1
               do bet=1, natot
                  do alp=1, natot
c                    rc is the relative lattice coordinate
                     ralp(:)=rc(:,alp)-rc(:,bet)+dble(ir(:)) 
                     ralp=matmul(crys%avec,ralp)
                     x2 =ralp(1)*ralp(1)+ralp(2)*ralp(2)+ralp(3)
     $                    *ralp(3)
                     x = sqrt(x2)
                     ep = exp(-eta*x2)
                     er = myderfc(seta*x)
                     xseta = x*seta
                     xseta2 = x2*eta

                     kfun = (1.d0+1.5d0/xseta2)*ep
     $                    + 3*sqrt(pi)/(4.d0*xseta*xseta2)*er
c     
c                       add to the tensor:
c     
c     
                     kfun = kfun*aconst*dipole(alp)
                     kfundivx2 = kfun/x2
                     kfunthird = kfun*third
                     do l=1,3
                        tens(l,:,bet)= tens(l,:,bet) 
     $                       + ralp(l)*ralp(:)*kfundivx2
                        tens(l,l,bet) = tens(l,l,bet) -kfunthird
                     end do
                  end do
               end do
            endif
            
         end do
      end do


      call all_sum_all(tens,natot*9) ! sum across all processors

c
c     treat the atoms inside the unit cell, i.e. R=0
c

      do bet=1, natot
         do alp=1, natot
            if(alp.ne.bet) then
               ralp(:)=rc(:,alp)-rc(:,bet)
               ralp=matmul(crys%avec,ralp)
               x2 =ralp(1)*ralp(1)+ralp(2)*ralp(2)+ralp(3)
     $              *ralp(3)
               x = sqrt(x2)
               ep = exp(-eta*x2)
               er = myderfc(seta*x)
               xseta = x*seta
               xseta2 = x2*eta
               
               kfun = (1.d0+1.5d0/xseta2)*ep
     $              + 3*sqrt(pi)/(4.d0*xseta*xseta2)*er
               kfun = kfun*aconst*dipole(alp)
               kfundivx2 = kfun/x2
               kfunthird = kfun*third
               do l=1,3
                  tens(l,:,bet)= tens(l,:,bet) 
     $                 + ralp(l)*ralp(:)*kfundivx2
                  tens(l,l,bet) = tens(l,l,bet) -kfunthird
               end do
            endif
         end do
      end do

      ewald_tens=0
      ewald_tens(:,:,1:natot) = dble(tens(:,:,1:natot))

      write(9,110)
      do i=1, crys%ntype
         write(9,120) crys%nameat(i), crys%dipole(i)
      end do

      if(iand(ioutput,8).eq.8) 
     $     write(9,940) gimmetime()-t0
      call myflush(9)
      
      deallocate(tens)
      deallocate(stfac)
      deallocate(dipole)

      deallocate(rc)

      return


 110  format(/' ATOM TYPE       MAGNETIC POLARIZABILITY IN A.U.'/)
 120  format(' ',a2,25x,f12.7)

 910  format(' ------------- EWALD DIPOLE TENSORS --------------')
 920  format(' ATOM NUMBER:', i5)
 930  format(3(10x,3f20.10/))

 940  format(/' TIME FOR EWALD DIPOLE CALCULATION:',f12.3)

      end
