c
c     @process extchk
      subroutine flevel(imode, ipr, pw_params, energs, 
     $     crys,bands,kpoints)

      include 'use.h'
      implicit none             ! implicit? No!
      include 'interface.h'

c     subroutine finds the fermi level and computes
c     nband, ifmax, and the (possibly fractional)
c     level occupancy for a metallic solid.
c
c     1996    Bernd Pfrommer, while at UC Berkeley
c      
c     based on the subroutine flevel from Alberto Garcia's pw program
c
c     for the spin-polarized case, the eigenvalues must be shifted
c     by vin(G=0), which is different for spin up/spin down. vin(G=0)
c     is v_xc(G=0), because G=0 for the Hartree- and ionic potential is
c     set to 0.
c     later in the charge subroutine, the fermi energy must be shifted back.
c     

c     ARGUMENTS:
c     ---------

      integer 
     $     imode,               ! mode of operation
     $     ipr                  ! print flag
      
      type (pw_parameter) pw_params 
c     get ioccup from pw_params
c            ioccup = 0 standard fermi level smearing
c            ioccup = 1 fill lowest ztot bands
c            ioccup = 2 fill bands antiferromagnetically
c     also get the fermi level smearing dsmear from here
c
      type(energy) energs       
c            input: energs%vxc0
c            ouput: energs%efermi

      type(crystal) crys
c            input: crys%ztot
c
      type(kpoint) kpoints

      type(band) bands          ! band structure info
c
c          input:  bands%energy
c          input:  bands%nband
c          output: bands%occup ! this is really 2*occup*weight(kpoint)
c          output: bands%ifmax


C     ---------------------- local variables ---------------------------

      double precision 
     $     emin, emins(2),sumt, sumw, xdum,
     $     pi, delta, range, ryd,
     $     sumwprime, deriv_contrib, derivative, 
     $     inv_smear, 
     $     dsmear,              ! gaussian smearing parameter in Rydbergs
     $     desm, fraction, weight_diff, myderfc,
     $     dnel     ! sum for total number of electrons

      external myderfc

      logical
     $     iafmag               ! flag if antiferromagnetic/magnetic

      integer 
     $     nrk,                 ! number of k-points
     $     ioccup,              ! occupy levels according to ioccup
     $     ispn,                ! number of spins: 1 or 2
     $     i, idum, j, nel, nelp, itcount, itmax, is, tnel, n

      parameter (
     $     itmax =  1000,       ! max number of iterations for efermi 
     $           pi    =  3.141592653589d0,
     $           range =  4.d0,     ! range beyond which erfc is uninteresting
     $           delta =  1.d-8,      
     $           ryd   =  13.60580d0)

      integer, allocatable ::
     $     ind(:), irk(:)

      double precision, allocatable ::
     $     el(:),               ! the eigenvalues as a contiguous array
     $     ws(:)                ! the weights of the k-points, for ispn=2


      external sort             ! external subroutines
c     --------------------------------------------------------------------
c
c     transfer data from the structures to local variables
c

      ioccup = pw_params%occupy

      if(ioccup.eq.2) then 
         iafmag=.true.
         ioccup=0               
      else
         iafmag=.false.
      endif

      nrk = bands%nrk
      ispn = bands%nspin
      dsmear = pw_params%smearing/ryd

c     --------------------------------------------------------------------
c                     allocate dynamic arrays

      nelp = nrk*bands%max*ispn    ! total number of electrons possible
      
      allocate(ind(nelp))
      allocate(irk(nelp))
      allocate(el(nelp)) ; allocate(ws(nrk*ispn))
c

      energs%evalshift=energs%vxc0

      nel = 0                   ! initialize some arrays....
      do is=1, ispn
         do i = 1, nrk
            do j = 1, bands%nband(i,is)
               irk(nel+j) = i
               el(nel+j) = bands%energy(j,i,is)+energs%vxc0(is)
            end do
            nel = nel + bands%nband(i,is)
            ws(i+nrk*(is-1)) = kpoints%w(i) ! copy weights over into temp array
         end do
      end do
c
c     sort the energy levels
c
      if(iafmag) then            ! sort spin up/down separately
         call sort(nel/2,el,ind)
         call sort(nel/2,el(1+nel/2),ind(1+nel/2))
      else
         call sort(nel,el,ind)
      endif
c
c     Naive section, straight counting.
c
c      sum up the weight factors until we reach the maximum number
c      allowed 
c

      sumt = crys%ztot/2.d0*dble(ispn)
      sumw = 0.d0

      do i = 1, nel
         sumw = sumw + ws(irk(ind(i)))
         if (sumw .gt. sumt+delta) exit ! leave the do loop
      end do

c
c        check if enough bands have been computed
c     
      if(sumw .gt. sumt-delta) then
         if(i.gt.nel)  then
            i=i-1
         else
            sumw = sumw - ws(irk(ind(i))) ! subtract overcounting 
         endif
      else
         if(i.gt.nel)  i=i-1
         write(9,600) 2.d0*(sumt-sumw)
         sumt = sumw
      endif
         
      emin = el(ind(i))-delta   ! new starting guess
c
      if (sumw .le. sumt-delta) then
c
c        fractional filling of a band
c
         fraction = (sumt-sumw)/ws(irk(ind(i)))
         if(ipr.gt.0) then
            write(9,9000) irk(ind(i)), fraction
 9000       format(/' Naive fractional occupancy at k point',i3,
     $           ', frac =',f8.6)
         endif
         emin = el(ind(i))
      end if

c
c     Use the estimate for Efermi to start a new
c     filling scheme with smearing.
c
c
      inv_smear = 1.d0 / dsmear

      energs%efermi(1) = emin
c
      itcount =0

 80   continue                  ! start of iterative loop
c
      itcount = itcount + 1

      sumw = 0.d0
      sumwprime = 0.d0
c
c     Disregard the sorting information.
c
      do i = 1, nel
            
         desm = (el(i)-energs%efermi(1)) * inv_smear
c
c     when the level is too far from the Fermi surface, act accordingly.
c
         if (abs(desm) .ge. range) then
            fraction = 1.d0
            if (desm .gt. 0) fraction = 0.d0
            deriv_contrib = 0.d0
         else
            fraction = 0.5d0 * myderfc(desm)
            deriv_contrib = exp(-desm*desm)
         endif
         sumw = sumw + fraction * ws(irk(i))
         sumwprime = sumwprime + deriv_contrib * ws(irk(i))
      end do
c
c     newton method steps...
c

      weight_diff = sumw - sumt
      derivative = sumwprime * 2 * inv_smear / sqrt(pi)
c
      if(abs(weight_diff) .gt. 1.d-9) then
         if(abs(derivative).gt.1d-20) then
            energs%efermi(1) = energs%efermi(1)-weight_diff
     $           /derivative
         else                   ! dirty hack
            energs%efermi(1) = energs%efermi(1)-weight_diff*0.1
         endif
         if(itcount.lt.itmax) go to 80
         write(9,*) 'flevel: not converged after ',itmax,' iterations!'
         stop 
      endif

      energs%efermi(2)=energs%efermi(1)
c
c
c     ---------- we have the fermi energy. now we determine the fracs ---
c
c
      if(imode.eq.0) then
         energs%efermi(1)=energs%inputfermi;
         energs%efermi(2)=energs%inputfermi;
      endif

      tnel = nel                ! total nel

      nel =0

      dnel =0.d0

      do is=1, ispn
         do i = 1, nrk
c     
            bands%ifmax(i,is) = 0
            do j=1,bands%nband(i,is)
c     
               nel = nel+1

               if(ioccup.eq.0) then
                  if(iafmag) then
                     desm=(el(nel)-energs%efermi(is))*inv_smear
                  else
                     desm=(el(nel)-energs%efermi(1))*inv_smear
                  endif
                  if (desm .lt. range) then
                     bands%ifmax(i,is) = j
                     bands%occup(j,i,is) = 2.d0*kpoints%w(i)
                     if (desm .gt. -1.d0*range) 
     $                    bands%occup(j,i,is)=1.d0*kpoints%w(i)
     $                    *myderfc(desm)
                  else
                     bands%occup(j,i,is) = 0.d0
                  endif
               else
                  if(dble(j).le.(crys%ztot+1d-6)/dble(ispn)) then
                     bands%ifmax(i,is) = j
                     bands%occup(j,i,is) =2.d0*kpoints%w(i)
                  else
                     bands%occup(j,i,is) =0.d0
                  endif
               endif
               dnel = dnel + bands%occup(j,i,is)*0.5d0
            end do
            
         end do
      end do
c
c
c
      if(iafmag) then
         if(ipr.gt.1)  write(9,9021) energs%efermi(1), 
     $        energs%efermi(2)
 9021    format( /' The Fermi level (includes vxc(0))',
     $        ' for spin(up,down) is ' //
     $        '(',f14.8,',',f14.8,') Ry')
      else
         if(ipr.gt.1)  write(9,9020) energs%efermi(1)
         if(ipr.gt.1)  write(9,9022) energs%vxc0(1)
         
 9020    format( /' The Fermi level (includes vxc(0)) is',f14.8,' Ry')
 9022    format( /' vxc0=',f14.8,' Ry')
      endif


      if(ioccup.eq.1) write(9,530)

      if(ipr.ge.1) then
       do is=1,ispn
          if((ispn.eq.2).and.(is.eq.1)) write(9,'(1x,a)') 'Spin up'
          if((ispn.eq.2).and.(is.eq.2)) write(9,'(1x,a)') 'Spin down'
          do i=1, nrk
             write(9,500) i
             do n=1,bands%nband(i,is),7
                write(9,510) (bands%energy(j,i,is)+energs%vxc0(is), 
     $               j=n,min(bands%nband(i,is),n+6))
                write(9,515) (bands%occup(j,i,is)/2.d0/kpoints%w(i), 
     $               j=n,min(bands%nband(i,is),n+6))
             end do
          end do
       end do
      endif

      deallocate(ind) ; deallocate(irk);
      deallocate(el); deallocate(ws);

      return
c
 500  format(' k-point:',i3)
 510  format(7f10.3)
c 510  format(7f22.16)
 515  format(3x,7(' (',f5.3,')  '))
c 515  format(3x,7(' (',f22.16,')  '))
 530  format('Fractional occupation numbers (fixed occupation!):')

 600  format(' *** WARNING: NOT ENOUGH BANDS ARE COMPUTED.',/
     $     ' *** THE CRYSTAL IS POSITIVELY CHARGED:',f12.6)
      end

