C*
C*
      subroutine generate_gspace(ioutput, gs, crys, symms)

      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
c      INPUT:
c      ------

      integer                         
     $     ioutput                  ! output flag

      type (parallel_gspace)   gs
      type (crystal)  crys
      type (symmetry) symms

c     ------------------------------------------------------------------
c
c     Multiprocessor version of the traditional gspace routine.
c     The gspace is now maintained differently: Each processor only
c     holds the particular area of the g-space, which is assigned to it.
c     
c     The layout of the data is dictated by the FFT: it is the 
c     only part which requires a substantial amount of communication
c     between the processors:
c
c       * each processor holds a certain number of rods,
c         which are lines along the z-axis of the FFT grid.
c         To get good load balancing, the rods are handed out
c         to processors in descending length:
c
c
c         +
c         ++
c         +++
c         ++++
c         +++++
c         ++++++
c         012210....
c
c         This idea is due to Andrew Canning.
c
c       * each processor holds only the data on those gvectors 
c         that fall on one of the rods it holds. In other words, the
c         ball of gvectors is pierced by the rods which point along the z
c         axis, and each processor holds those G-vectors that fall on the
c         rods which it owns. The map between gvectors and rods is done
c         in the array  gs%order. It is used to unfold the data
c         when FFTs are performed, and is also necessary whenever a loop
c         over gspace is performed.
c

c
c     The routine to collect the gvectors into stars was written by JLM.
c
c     All the other stuff was written by  Bernd Pfrommer, UC Berkeley
c
c
c
c     ------------------------- actions ---------------------------------
c
c     *  generates the gspace according to the variable 
c                gs%gmax
c                gs%rk(3)  (the k-point)
c                gs%nproc
c                gs%myproc
c     *  if the fourier grid dimensions gs%fftsize  are zero, they 
c        get automatically set to snugly hold the sphere
c        corresponding FFT in the fftstruc 
c
c     *  if
c           gs%igvec    generate the gvectors (h,k,l) 
c           gs%iekin    compute the kinetic energy of the g-vectors
c           gs%istar    generate the star information: inds, mstar, phase
c           gs%iinverse find the inverse of each gvec (local gspace only)
c           gs%struc    compute the structure factors, and determine if 
c                       crystal has inversion symmetry
c
c     * the gs%order array is of great importance. It contains
c       the mapping between the packed gspace and the FFT grid
c
c     gs%order(1,i)=irod   gives the rod number of the FFT rod owned by
c     gs%myproc, on which the data i belongs. The numbering starts
c     from 0. If you know the rod number, the fft grid position
c     is given by: 
c       
c           nx = irod/gs%fftsize(2)
c           ny = mod(irod, gs%fftsize(2))
c
c
c     ^ y
c     |
c     ++++++++++++++++++++++++++++++++++
c     |  |  |  |  |  |  |  |  |  |  |  |
c     ++++++++++++++++++++++++++++++++++
c     |  |  |  |  |  |  |  |  |  |  |  |
c     ++++++++++++++++++++++++++++++++++
c     |  |  |  |  |  |  |  |  |  |  |  |
c     ++++++++++++++++++++++++++++++++++
c     |  |  |  |  |  |  |  |  |  |  |  |
c     ++++++++++++++++++++++++++++++++++
c     |  |  |  |  |  |  |  |  |  |  |  |
c     ++++++++++++++++++++++++++++++++++
c     |  |  |  |  |  |  |  |  |  |  |  |
c     G++G++++++++++++++++++++++++++++++
c     |  |  |  |  |  |  |  |  |  |  |  |
c     G++G++G+++++++++++++++++++++++++++
c     |  |  |  |  |  |  |  |  |  |  |  |
c     G++G++G++G++++++++++++++++++++++++
c     |  |  |  |  |  |  |  |  |  |  |  |
c     G++G++G++G++++++++++++++++++++++++ ---> x
c
c
c     
c
c     ------------------------ memory allocation ------------------------
c
c     this subroutine allocates memory, and attaches it to various
c     structures:
c
c     * always
c
c     gs%order       (the order array)
c     gs%fastfind    (a small index array for faster finding)
c
c
c     * additionally, if
c
c           gs%igvec    ==>  gs%gvec
c           gs%iekin    ==>  gs%ekin
c           gs%istar    ==>  gs%inds, gs%phase, gs%mstar
c           gs%iinverse ==>  gs%inverse
c           gs%istruc   ==>  gs%struc

c     ------------------ local variables ------------------------------

      double precision 
     $     qk(3), pi, 
     $     gmod, gmax2, rgm, rkmod, 
     $     fi, gmr, diff, delta, gimmetime, t0
       
      complex(kind=8)
     $     ph, ex(48), str

      integer 
     $     ns,                  ! number of stars
     $     i,  j,  k,           ! g-space integers
     $     l,ir,ih,
     $     ncount,              ! counts number of symmops leaving G invariant
     $     ktran(3,48),         ! symmetry transformed vector
     $     ngt,                 ! number of vectors in total gspace
     $     ng,                  ! number of vectors on myproc
     $     myblocks,
     $     kstart, nrods,
     $     ntotrod,             ! total number of rods everywhere
     $     ib,                  ! counts blocks
     $     ip,                  ! counts planes within one block
     $     iprot, iavail, itest,
     $     kk(3), kmax(3), nx,ny,p,
     $     iproc, lorder, findvec,
     $     nt, ja, jmax, igv(3), ili,
     $     sflag                ! detect if inside sphere

c
c     variables for the generic gspace loop
c
      integer il, fftn(4),ffth(4),iorder,irod,gv(4),gv3


      parameter(delta = 1.0d-6)

      integer, allocatable :: gtvec(:,:), ihelp(:), rodmap(:,:),
     $     rodlen(:), idxarr(:)
      double precision, allocatable :: ekt(:)

      logical 
     $     imyrod, iflag

      external findbound, findvec, gimmetime

c     -----------------------------------------------------------------
c     calculate length of k-vector

      pi = 4.d0*atan(1.d0)

      gs%ntype = crys%ntype

      t0=gimmetime()
      rkmod = sqrt(dot_product(gs%rk,matmul(crys%bdot,gs%rk)))

c     determine boundaries kmax such that grid holds the sphere snugly

      call findbound(gs%gmax+rkmod,crys%bvec,kmax(1),kmax(2),kmax(3))

c     
c     Pick nice FFT grid if it has not already been picked
c
c
      if((gs%fftsize(1).le.0).or.
     $     (gs%fftsize(2).le.0).or.(gs%fftsize(3).le.0)) then 
         gs%fftsize(1:3)=2*kmax(1:3)+1
         call adjustfft(gs%fftsize,symms)
      endif

      gs%fftsize(4) = kmax(1)   ! right limit as is
      gs%fftsize(5) = mod(gs%fftsize(1)-kmax(1),gs%fftsize(1))

c     sanity checks: sphere should not be larger than grid limits:

      if((2*kmax(1)+1.gt.gs%fftsize(1)).or.
     $     (2*kmax(2)+1.gt.gs%fftsize(2))
     $     .or.(2*kmax(3)+1.gt.gs%fftsize(3))) then
         write(9,*) ' sphere does not fit into FFT grid!'
         write(9,*) ' gmax:', gs%gmax
         write(9,*) ' nfft:', gs%fftsize
         write(9,*) ' kmax:', kmax
         write(9,*) ' presumably, the cutoff is too low, and |k| is ',
     $        'comparable to gmax'
         stop
      endif

c
c     compute the size of the FFT grid that this processor holds
c     in realspace
c
      gs%r_size = (gs%fftsize(2)*gs%fftsize(3))/gs%nproc

      if( mod(gs%fftsize(2)*gs%fftsize(3),gs%nproc).gt.gs%myproc)
     $     gs%r_size=gs%r_size+1
      gs%r_size=gs%r_size*gs%fftsize(1)

      gs%ig0 =-1                ! will be set to >0 if myproc holds G=0

      gmax2 = gs%gmax*gs%gmax

c
c     ------- count the number of elements in total and small gspace. -------
c                     set up the array with rod lengths
c

      gs%lorder = 0
      gs%length = 0
      gs%totlength  = 0 
      
      ntotrod = 0
      allocate(rodlen(gs%fftsize(1)*gs%fftsize(2)))
      allocate(rodmap(gs%fftsize(2),gs%fftsize(1)))
      rodmap = 0
      allocate(idxarr(gs%fftsize(1)*gs%fftsize(2)))

      do i=-kmax(1), kmax(1)
         qk(1) = dble(i) + gs%rk(1)
         nx=mod(i+gs%fftsize(1),gs%fftsize(1))
         do j=-kmax(2), kmax(2)
            qk(2) = dble(j) + gs%rk(2)
            ny=mod(j+gs%fftsize(2),gs%fftsize(2))
            sflag = 0
            do k=-kmax(3), kmax(3)
               qk(3) = dble(k) + gs%rk(3)

c              calculate norm with special metric

               gmod = (qk(1)*crys%bdot(1,1) + qk(2)*crys%bdot(2,1) +
     $              qk(3)*crys%bdot(3,1))*qk(1) +
     $              (qk(1)*crys%bdot(1,2) + qk(2)*crys%bdot(2,2) +
     $              qk(3)*crys%bdot(3,2))*qk(2) +
     $              (qk(1)*crys%bdot(1,3) + qk(2)*crys%bdot(2,3) +
     $              qk(3)*crys%bdot(3,3))*qk(3)
               
               if (gmod .le. gmax2) then !we are inside the sphere
                  
                  if (sflag .eq. 0) then ! mark where we intruded sphere
c                    write(9,*) 'intruded sphere'
                     kstart = k
                     sflag = 1
                     ntotrod=ntotrod+1
                     rodmap(ny+1,nx+1) = ntotrod
                  endif
                  gs%totlength = gs%totlength + 1
               else             ! we are outside the sphere
                  if(sflag.eq.1) then ! we have left the sphere
                     sflag = 0
                     rodlen(ntotrod) = k-kstart
                  endif
               endif

            end do              ! loop along 3rd dimension
            if(sflag.eq.1) then ! reached border of sphere
               sflag=0
               rodlen(ntotrod) = k-kstart
            endif

         end do                 ! 2nd dim
      end do                    ! 1st dim
c
c     sort array with rod lengths in ascending order
c
      do i=1,ntotrod
         idxarr(i)=i
      end do
      call iheapsort(ntotrod,rodlen,idxarr)

c
c     figure out how many rods this processor holds, and allocate array
c
      nrods = ntotrod/gs%nproc

      if(mod(ntotrod/gs%nproc,2).eq.0) then
         if(mod(ntotrod,gs%nproc).gt.gs%myproc) nrods=nrods+1
      else
         if(mod(ntotrod,gs%nproc).ge.gs%nproc-gs%myproc) nrods=nrods+1
      endif
      gs%lorder=nrods
c      write(9,*) 'total number of rods:',gs%lorder
c      
c     assign rods to processors. modify rodmap. 
c
c     this scales like N^4/3, so watch out for large systems
c
      gs%length=0
      do i=0, ntotrod-1
         if(((mod(i/gs%nproc,2).eq.0).and.
     $        (mod(i,gs%nproc).eq.gs%myproc)).or.
     $        ((mod(i/gs%nproc,2).ne.0).and.
     $        (mod(i,gs%nproc).eq.(gs%nproc-gs%myproc-1)))) then
            gs%length=gs%length+rodlen(idxarr(ntotrod-i))
c            write(9,*) 'hold:', i+1,rodlen(idxarr(ntotrod-i))
            do j=1,gs%fftsize(1)
               do k=1, gs%fftsize(2)
                  if(rodmap(k,j).eq.idxarr(ntotrod-i)) then
                     rodmap(k,j)=-1 ! mark as belonging to this proc
c                     write(9,*) 'marked rod as mine'
                  endif
               end do
            end do
         endif
      end do
c
c     allocate arrays
c
      allocate(gs%order(3,gs%lorder))
      allocate(gs%fastfind(3,gs%fftsize(1)))
      gs%fastfind(:,:) = -1  ! invalidate



      if(gs%iekin) then ;  allocate(gs%ekin(gs%length))
      else ; nullify(gs%ekin); endif

      if(gs%igvec) then ; allocate(gs%gvec(3,gs%length))
      else ; nullify(gs%gvec) ; endif
            
      if(gs%istruc) then ; allocate(gs%struc(gs%ntype,gs%length))
      else ;  nullify(gs%struc) ; endif
      if(gs%istar) then
c        permanent ones, attached to gspace structure
         allocate(gs%inds(gs%length))
         allocate(gs%phase(gs%length))
         allocate(gs%mstar(gs%totlength))
c        volatile work arrays
         allocate(gtvec(3,gs%totlength))
         allocate(ihelp(gs%totlength))
         allocate(ekt(gs%totlength))
      else
         nullify(gs%mstar); nullify(gs%inds);  nullify(gs%phase);
      endif

c
c     ---------------- now really generate the gspace--------------
c
      if(gs%istruc) crys%icomplex=.false.

      lorder = 0
      ng = 0
      ngt = 0 

      do i=-kmax(1), kmax(1)
         qk(1) = dble(i) + gs%rk(1)
         nx=mod(i+gs%fftsize(1),gs%fftsize(1))
         do j=-kmax(2), kmax(2)
            qk(2) = dble(j) + gs%rk(2)
            ny=mod(j+gs%fftsize(2),gs%fftsize(2))
            sflag =0
c            write(9,*) nx,ny,'rodmap:', rodmap(ny+1,nx+1)
            imyrod=.false.
            if(rodmap(ny+1,nx+1).lt.0) then ! aha, it is me who holds it.
               imyrod=.true.    ! set flag
               if(gs%fastfind(1,nx+1).lt.0) then ! started new xy plane
                  gs%fastfind(1,nx+1)=lorder+1 
                  gs%fastfind(3,nx+1)=ng
               endif
               gs%fastfind(2,nx+1)=lorder+1 ! catch end of xy plane
            endif
            
            do k=-kmax(3), kmax(3)
               qk(3) = dble(k) + gs%rk(3)

c              calculate norm with special metric

               gmod = (qk(1)*crys%bdot(1,1) + qk(2)*crys%bdot(2,1) +
     $              qk(3)*crys%bdot(3,1))*qk(1) +
     $              (qk(1)*crys%bdot(1,2) + qk(2)*crys%bdot(2,2) +
     $              qk(3)*crys%bdot(3,2))*qk(2) +
     $              (qk(1)*crys%bdot(1,3) + qk(2)*crys%bdot(2,3) +
     $              qk(3)*crys%bdot(3,3))*qk(3)
               
               if (gmod .le. gmax2) then 

c     -------- we are inside the sphere
                  
                  if (sflag .eq. 0) then ! mark where we intruded sphere
c                    write(9,*) 'intruded sphere'
                     sflag = 1
                     if(imyrod) then
                        lorder = lorder+1
                        gs%order(1,lorder) = 
     $                       nx*gs%fftsize(2)+ny
                        gs%order(2,lorder) = 
     $                       mod(k+gs%fftsize(3),gs%fftsize(3))
                     endif
                  endif
                  if(imyrod)  then
                     ng =  ng + 1
                     if((i.eq.0).and.(j.eq.0).and.(k.eq.0))
     $                    gs%ig0 = ng ! i hold the G=0 vector
                  endif
                  if(gs%istar) ngt = ngt + 1

                  if(gs%igvec.and.imyrod) then
                     gs%gvec(1,ng)=i
                     gs%gvec(2,ng)=j
                     gs%gvec(3,ng)=k
c                     write(9,*) ng, nx,ny,gs%gvec(:,ng)
                  endif
                  if(gs%istar) then
                     gtvec(1,ngt)=i
                     gtvec(2,ngt)=j
                     gtvec(3,ngt)=k
                     ekt(ngt)=gmod
                     if(imyrod) then
                        ihelp(ngt)=ng
                     else
                        ihelp(ngt)=0
                     endif
                  endif

                  if(gs%iekin.and.imyrod) then
                     gs%ekin(ng)=gmod
                  endif

                  if(gs%istruc.and.imyrod) then
                     do nt=1,crys%ntype ! loop over atomic types
                        str = (0.d0,0.d0)
                        jmax = crys%natom(nt)
                        do ja=1,jmax
                           fi = dble(i)*crys%rat(1,ja,nt) +
     $                          dble(j)*crys%rat(2,ja,nt) +
     $                          dble(k)*crys%rat(3,ja,nt)
                           str = str + cmplx(cos(fi),-sin(fi),kind=8)
                        end do

                        if(abs(aimag(str)).gt.1d-6) 
     $                       crys%icomplex=.true.
                        gs%struc(nt,ng) = str
                     end do
                  endif

c     -------- we are outside the sphere
               else
                  if(sflag.eq.1) then
                     if(imyrod) then
                        gs%order(3,lorder)= 
     $                       mod(k-1+gs%fftsize(3),gs%fftsize(3))
                     endif
c                    write(9,*) 'left sphere'
                     sflag = 0
                  endif
               endif

            end do              ! loop along 3rd dimension
            if(sflag.eq.1) then ! didn't leave the sphere
               sflag =0
               if(imyrod) gs%order(3,lorder)=
     $              mod(k-1+gs%fftsize(3), gs%fftsize(3))
            endif
         end do                 ! 2nd dim

      end do                    ! 1st dim


c     -------------------------------------------------------------------
c                          set up stars if required
c

      if(gs%istar) then
         
c
c        sort the g-vectors according to length and collect
c        heapsort algorithm from numerical recipes,
c        original coding by JLM

         if(ngt.gt.1) then

         l=ngt/2+1
         ir=ngt

 20      continue
         if(l.gt.1)then
            l=l-1
            rgm=ekt(l)
            kk(1:3)=gtvec(1:3,l)
            ih = ihelp(l)
         else
            rgm=ekt(ir)
            kk(1:3)=gtvec(1:3,ir)
            ih = ihelp(ir)
            ekt(ir)=ekt(1)
            gtvec(1:3,ir)=gtvec(1:3,1)
            ihelp(ir)=ihelp(1)
            ir=ir-1
            if(ir.eq.1)then
               ekt(1)=rgm
               gtvec(1:3,1)=kk(1:3)
               ihelp(1)=ih
c     
               goto 25
c
            endif
         endif
         i=l
         j=l+l
 21      if(j.le.ir)then
            if(j.lt.ir)then
               if(ekt(j).lt.ekt(j+1)) j=j+1
            endif
            if(rgm.lt.ekt(j))then
               ekt(i)=ekt(j)
               gtvec(1:3,i)=gtvec(1:3,j)
               ihelp(i)=ihelp(j)
               i=j
               j=j+j
            else
               j=ir+1
            endif
            go to 21
         endif
         ekt(i)=rgm
         gtvec(1:3,i)=kk(1:3)
         ihelp(i)=ih
         go to 20
 25      continue

         endif                     ! end of if(ngt.gt.1)...
c
c        Collect G-vectors into stars and compute "phase" factors 
c                   First star is (0,0,0)
c
         ns = 1
         gs%mstar(1) = 1
         if(ihelp(1).gt.0) then
            gs%inds(ihelp(1))=1
            gs%phase(ihelp(1))=(1.d0, 0.d0)
         endif
c
         iprot = 1
         iavail = 2
         itest = 2
c
 30      continue
c
         if(iavail .gt. ngt) goto 50
c
         if(itest .gt. ngt) then
            diff = 1.0
         else
            diff = abs(ekt(itest)-ekt(iprot))
         endif
         if(diff .gt. delta) then
c
c        new star. calculate all gvectors in star
c
            iprot  = iavail
            iavail = iprot + 1
            itest  = iprot + 1
            ns = ns + 1
            gs%mstar(ns) = 1
            do i=1,symms%ntrans
               ktran(1,i) = symms%mtrx(1,1,i)*gtvec(1,iprot) + 
     +              symms%mtrx(1,2,i)*gtvec(2,iprot) +
     +              symms%mtrx(1,3,i)*gtvec(3,iprot)
               ktran(2,i) = symms%mtrx(2,1,i)*gtvec(1,iprot) + 
     +              symms%mtrx(2,2,i)*gtvec(2,iprot) +
     +              symms%mtrx(2,3,i)*gtvec(3,iprot)
               ktran(3,i) = symms%mtrx(3,1,i)*gtvec(1,iprot) +
     +              symms%mtrx(3,2,i)*gtvec(2,iprot) +
     +              symms%mtrx(3,3,i)*gtvec(3,iprot)
            end do
            do i=1,symms%ntrans
               fi = symms%tnp(1,i)*dble(ktran(1,i)) +
     +              symms%tnp(2,i)*dble(ktran(2,i)) +
     +              symms%tnp(3,i)*dble(ktran(3,i))
               ex(i) = cmplx(cos(fi),sin(fi),kind=8)
            end do
            ph = 0
            ncount = 0
            do i=1,symms%ntrans
               if ((ktran(1,i)-gtvec(1,iprot) .eq. 0) .and.
     $              (ktran(2,i)-gtvec(2,iprot) .eq. 0) .and.
     $              (ktran(3,i)-gtvec(3,iprot) .eq. 0)) then
                  ph = ph + ex(i)
                  ncount = ncount + 1
               endif
            end do
            if(ihelp(iprot).gt.0) then
               gs%phase(ihelp(iprot))=ph/dble(ncount)
               gs%inds(ihelp(iprot))=ns
            endif
c
            goto 30
c
         endif
c
         iflag = .false.
         ncount = 0
         ph = 0
c
c        calculate phase factor for gvec(itest)
c
         do i=1,symms%ntrans
            if ((ktran(1,i)-gtvec(1,itest) .eq. 0) .and.
     $           (ktran(2,i)-gtvec(2,itest) .eq. 0) .and.
     $           (ktran(3,i)-gtvec(3,itest) .eq. 0)) then
               iflag = .true.
               ph = ph +ex(i)
               ncount = ncount + 1
            endif
         end do
         if(iflag) then         ! gvec(itest) is related to prototype
            gs%mstar(ns) = gs%mstar(ns) + 1
            if(itest .ne. iavail) then
               kk(1) = gtvec(1,iavail)
               kk(2) = gtvec(2,iavail)
               kk(3) = gtvec(3,iavail)
               gtvec(1,iavail) = gtvec(1,itest)
               gtvec(2,iavail) = gtvec(2,itest)
               gtvec(3,iavail) = gtvec(3,itest)
               gtvec(1,itest) = kk(1)
               gtvec(2,itest) = kk(2)
               gtvec(3,itest) = kk(3)
               gmr = ekt(iavail)
               ekt(iavail) = ekt(itest)
               ekt(itest) = gmr
               ih  = ihelp(iavail)
               ihelp(iavail) = ihelp(itest)
               ihelp(itest) = ih
            endif
            if(ihelp(iavail).gt.0) then
               gs%phase(ihelp(iavail))=ph/dble(ncount)
               gs%inds(ihelp(iavail))=ns
c               write(9,*) gtvec(:,iavail), ph/dble(ncount)
            endif
            iavail = iavail + 1
         endif
         itest = itest + 1
c
         goto 30
 50      continue

         gs%nstar = ns

      endif

      if(gs%istar) then
         deallocate(gtvec)
         deallocate(ihelp)
         deallocate(ekt)
      endif

c
c     find gspace inversion information if requested
c
      if(gs%iinverse) then
         if(gs%nproc.gt.1) then
            write(9,*) 'inverse array only for LOCAL gspaces!'
            stop
         endif

         allocate(gs%inverse(gs%length))
         
         il=0
         fftn(1:3)=gs%fftsize(1:3)
         fftn(4)=gs%fftsize(3)
         ffth(:)=fftn(:)/2
         do iorder=1, gs%lorder ! loop through x/y gspace 
            irod=gs%order(1,iorder)
            gv(1)= irod/gs%fftsize(2)
            gv(2)= mod(irod,gs%fftsize(2))
            gv(3)= gs%order(2,iorder)
            gv(4)= gs%order(3,iorder)
            gv(:)= mod(gv(:)+ffth(:),fftn(:))-ffth(:) 
            do gv3=gv(3),gv(4)  ! loop over z axis
               il = il + 1
               igv(1)=-gv(1)
               igv(2)=-gv(2)
               igv(3)=-gv3
               ili = findvec(igv,gs)
               if(ili.le.0) then
                  write(9,*) 'generate_gspace: cannot find inverse!'
                  stop
               endif
               gs%inverse(il)=ili
            end do
         end do
      else
         nullify(gs%inverse)
      endif
c
c     set up the packing information arrays for the parallel 3dfft
c

      call setup_packinfo(gs)
      call print_gspace(0,gs)


      deallocate(rodlen)
      deallocate(rodmap)
      deallocate(idxarr)

      if(iand(ioutput,8).eq.8) 
     $     write(9,940) gimmetime()-t0

      call myflush(9)
      


      return


 940  format(' TIME FOR GSPACE SETUP:',f12.3)


      END

