c ----------------------------------------------------------------------
c
c   computes the magnetic susceptibility in the LDA by perturbation theory
c
c   (1996)      Bernd Pfrommer and Francesco Mauri
c               UC Berkeley
c
c   ----------------------------------------------------------------------
c     @process extchk

      subroutine magnetic_suscept(qmag, g0mask,
     $     pw_params, ffts, bands,
     $     pot_gspace, vion, crys, syms, pspot,
     $     kpoints, chi_g)

      use all_to_all_module
      include 'use.h'
      implicit none 
      include 'interface.h'
      include 'all_to_all.h'

      type (kpoint) kpoints     ! the kpoints
      type (pw_parameter) pw_params ! energy cutoffs etc
      type (parallel_gspace) pot_gspace
      type (fftstruc) ffts
      type (crystal) crys
      type (band) bands
      type (symmetry) syms
      type (pseudo_potential) pspot
      type (complex_gspace_array) vion ! the screened ionic potential in gspace
      double precision
     $     g0mask(3,3),           ! tensor to mask G=0 component
     $     qmag                 ! magnitude of q
c
c     OUTPUT:
c     ------
c
      complex(kind=8) chi_g(pot_gspace%length,3,3)
c
c
c     DESCRIPTION:
c     -----------
c
c     computes the diamagnetic susceptibility of insulators.
c     the loop structure is as follows
c
c
c    rho = 0
c    loop over k (kpoints)
c	generate Gspace for k
c       setup nonlocal derivative dV/dk @ k
c*	compute u_k 
c	pack    u_k
c	loop over q^  (wavevector of the applied magnetic field)
c	    loop over +-q
c		setup H_k+q
c               setup nonlocal derivative dV/dk @ k+q
c		compute extended starting guess for u_k+q
c
c*		compute u_k+q
c		loop over the two directions b0 perpendicular to q
c                    p0= q^ x b0 
c		     solve for u~_k+q,p0
c		     loop over i ( occupied bands)
c		        Ftrans u_k,i

c			loop over p1
c			   apply p1 to u~_k+q,p0,i
c			   fourier transform u~_k+q,p0,p1,i
c			   rho(r,b0,p1)= rho+sign(q)*u_k,i(r)*u~_k+q(r),p0,p1,i
c			end loop over p1
c		     end loop over i
c		loop over b0
c	    end loop over +-q
c	end loop over q
c    end loop over k


c     -------------- local variables --------------------------------
c
      integer
     $     p0, p1,alf,          ! counters for directions of momentum operator
     $     iq,                  ! qpoint counter
     $     nsub,                ! number of plane waves in subspace
     $     i,j,                 ! counter for occupied bands
     $     irkstart,            ! which kpoint to start with
     $     irk                  ! kpoint counter

      integer dq(3,3)
      data dq/1,0,0, 0,1,0, 0,0,1/
      
      type (parallel_gspace) 
     $     k_gspace,            ! the gspace for a given k
     $     ksub_gspace           ! the subspace for a given k


      type (hamiltonian)
     $     ham
      type (dense_hamiltonian)  ! the subspace hamiltonians 
     $     densham,             ! for k
     $     denshamkq            ! for k+q

      double precision 
     $     t0, tk, gimmetime, chi_avg,
     $     pi2,                 ! two times pi
     $     rk(3),               ! unshifted kpoint
     $     kq(3),               ! shifted kpoint k+q
     $     chi_macro(3,3),      ! g(G=0, q.ne.0), in ppm/mol [cgs]
     $     binverse(3,3)        ! inverse of the lattice vectors

      complex(kind=8), pointer ::
     $     u_kq(:,:),           ! the wave functions for a given k+q
     $     u_k(:,:),            ! the wave functions for a given k
     $     nloc_deriv_k(:,:,:,:), !  derivative of nonlocal potential at k
     $     nloc_deriv_kq(:,:,:,:), ! same, but at k+q
     $     phi(:), phi1(:),     ! temporary storage
     $     vloc(:),             ! local potential in realspace
     $     rhom_r(:,:,:),       ! rho_magnetic in realspace rhom_r(r,a0,a1)
     $     u_ktil(:,:,:),        ! u_ktilde in gspace
     $     u_kqtil(:,:),        ! u_kqtilde in gspace
     $     graduk(:,:),         ! the gradient of uk
     $     u_kqtil_r(:),        ! u_kqtilde in rspace
     $     u_k_rspace(:)        ! u_k in real space

      complex(kind=8)  
     $     delchi,
     $     chivv(3,3),          ! g(G=0, q.ne.0)
     $     chihh(3,3),          ! g(G=0, q.ne.0)
     $     kk_termvv(3,3),      ! g(G=0, q=0)
     $     kk_termhh(3,3)       ! g(G=0, q=0)
c     $     tsum(3), tsum1(3)

      double precision, allocatable :: 
     $     e_kq(:)              ! the eigenvalues at u_k+q


      logical idiaconv          ! if diagonalization has converged

      integer 
     $     b0, b1, iperm0, iperm1, iqsign, iguess, 
     $     n_ukq                ! number of computed n_ukq. Used as dummy.
      
      external gimmetime

c     ----------------------------------------------------------------
c*$*OPTIMIZE(0)


      pi2 = 8.d0*datan(1.d0)

      ham%ekinmod=pw_params%ekinmod 
      iguess = 1
      if(iand(pw_params%optimize,4).eq.4) iguess =0

      if(pw_params%epsdiag.gt.1d-12) write(9,150)
     $     pw_params%epsdiag

      binverse=transpose(crys%avec)/pi2
c
c     set up the local potential, which is the same for all kpoints
c
      allocate(vloc(ffts%r_size))
      call setup_local_potential(1,ffts,pot_gspace,vion%data(1,1,1),
     $     vloc(1))
c
c     allocate some other arrays
c
      allocate(rhom_r(ffts%r_size,3,3))
      rhom_r=0 
      chivv=0
      chihh=0
      allocate(e_kq(bands%max))

      irk=0
c
c     read checkpoint file if requested
c
      if(iand(pw_params%input,4).eq.4) 
     $        call chkpt_chi(irk,rhom_r(1,1,1),9*pot_gspace%r_size,
     $        pot_gspace%myproc, chivv,chihh, pw_params%output,2)

      irkstart=irk+1

      do irk=irkstart, kpoints%nrk
         tk=gimmetime()
         rk=kpoints%rk(:,irk)
         write(9,800) irk, rk
         call myflush(9)
c
c     set up the sub-gspace and the gspace for a given kpoint irk
c

         k_gspace%gmax=sqrt(pw_params%emax)
         k_gspace%rk=kpoints%rk(:,irk)
         k_gspace%nproc=pot_gspace%nproc; ! truly parallel
         k_gspace%name='kmag'
         k_gspace%myproc=pot_gspace%myproc; 
         k_gspace%fftsize=pot_gspace%fftsize
         k_gspace%istar=.false. ; k_gspace%igvec=.false.
         k_gspace%iekin=.true. ; k_gspace%iinverse=.false.
         k_gspace%istruc=.false.
         k_gspace%ipackinfo=.true.

         call generate_gspace(pw_params%output,k_gspace, crys, syms)
c       
c     allocate space for wave function u_k
c
         allocate(u_k(k_gspace%length,bands%max))
         allocate(u_kq(k_gspace%length,bands%max))

         allocate(phi(k_gspace%length))
         allocate(phi1(k_gspace%length))

         ksub_gspace%gmax=sqrt(pw_params%emaxsub)
         ksub_gspace%rk=k_gspace%rk ! inherit kpoint from big brother
         ksub_gspace%nproc=1;    ! scatter across all processors
         ksub_gspace%name='kmagsub'
         ksub_gspace%myproc=0; ksub_gspace%fftsize=(/0,0,0,0,0/)
         ksub_gspace%istar=.false. ; ksub_gspace%igvec=.true.
         ksub_gspace%iekin=.true. ; ksub_gspace%iinverse=.false.
         ksub_gspace%istruc=.false.
         ksub_gspace%ipackinfo=.false.

         call generate_gspace(pw_params%output,ksub_gspace, crys, syms)

c
c        set up the nonlocal part of H and the derivative for given k
c

         call create_hamiltonian(ham,k_gspace,pspot,vloc,ffts)

         t0=gimmetime()
         call setup_nonlocal_potential(ham,pspot,crys)
         if(iand(pw_params%output,8).eq.8) 
     $        write(9,922) gimmetime()-t0
c
c        generate starting guess for u_k
c
         call create_dense_hamiltonian(densham,ksub_gspace)

         call start_vectors(0,pw_params,
     $        ham, densham, bands%energy(1,irk,1), bands%max,
     $        u_k(1,1),  pot_gspace, ksub_gspace, 
     $        vion%data(1,1,1), crys)

c
c        compute the u_k
c
         bands%nband(irk,1)=bands%nband(irk,1)-1
         if(pw_params%bandgap.gt.0) then
            bands%nband(irk,1)=bands%min
            bands%energy(bands%min+1,irk,1)=
     $           bands%energy(bands%min,irk,1)+pw_params%bandgap
         endif
         call diagonalize_hamiltonian(pw_params,
     $        pw_params%maxitdiag, idiaconv, bands%min,
     $        bands%nband(irk,1),pw_params%epsdiag,
     $        bands%energy(1,irk,1),ham, u_k(1,1))
c
c        setup nonlocal derivative at k
c
         if(iand(pw_params%optimize,1).ne.1)  then
            allocate(nloc_deriv_k(k_gspace%length+pspot%nanl, 
     $           pspot%nanl,2,3)) 
            call setup_nonlocal_derivative(pw_params%output,qmag,
     $           rk(1),pspot, k_gspace, nloc_deriv_k(1,1,1,1), crys)
         else
            allocate(nloc_deriv_k(1,1,1,1))
         endif

c
c        compute for the case of q=0
c
         if(iand(pw_params%optimize,4).eq.4) then
            allocate(u_ktil(ham%gspace%length,bands%ifmax(irk,1),3))
         else
            allocate(u_ktil(ham%gspace%length,bands%ifmax(irk,1),1))
         endif
         

         call magnetic_kkterm(ham, densham, 
     $        nloc_deriv_k(1,1,1,1), qmag, rk(1), pspot,
     $        u_k(1,1), ham%gspace%length, ksub_gspace,
     $        bands%energy(1,irk,1), bands%ifmax(irk,1), 
     $        pw_params, crys, 1.d0, u_ktil(1,1,1), kk_termvv(1,1), 
     $        kk_termhh(1,1))
c
         if(iand(pw_params%optimize,4).ne.4) deallocate(u_ktil)
c
c        free to optimize for memory. then recompute below
c
         if(iand(pw_params%optimize,1).eq.1)  deallocate(nloc_deriv_k)

c
c        now compute u_k+q
c

         call create_dense_hamiltonian(denshamkq,ksub_gspace)

         do iq=1, 3             ! loop over all q-points
         do iqsign=1,-1,-2       ! sign of q   
            kq=kpoints%rk(:,irk)+
     $           dble(iqsign)*qmag*matmul(binverse,dq(:,iq))
c
c         Generate starting guess for u_k+q. Use same g-space as for u_k,
c         but modify its kinetic energy appropriately. Since the hamiltonian
c         refers to that g-space, it is altered implicitly, too.
            
            k_gspace%rk=kq; ksub_gspace%rk=kq;
            call compute_ekin_gspace(k_gspace,crys)
            call compute_ekin_gspace(ksub_gspace,crys)

            t0=gimmetime()
            call setup_nonlocal_potential(ham,pspot,crys)
            if(iand(pw_params%output,8).eq.8) 
     $           write(9,922) gimmetime()-t0

            if(iand(pw_params%optimize,2).eq.2) then ! dont compute start guess
               u_kq=u_k
               nsub=densham%gspace%length
               denshamkq%neig=densham%neig
               if(.not.associated(denshamkq%matrix)) then
                  allocate(denshamkq%matrix(size(densham%matrix)))
               endif
               denshamkq%matrix(:)=densham%matrix(:)
               denshamkq%energy(1:nsub)=densham%energy(1:nsub) 
               e_kq(1:bands%max)=densham%energy(1:bands%max)
               n_ukq = bands%min
            else
               call start_vectors(0, pw_params, ham,
     $              denshamkq, e_kq(1), bands%max,
     $              u_kq(1,1),   pot_gspace, 
     $              ksub_gspace, vion%data(1,1,1), crys)
               n_ukq = bands%max-1
            endif
c
c           compute the u_k+q
c
            if(pw_params%bandgap.gt.0) then
               n_ukq=bands%min
               e_kq(bands%min+1)=e_kq(bands%min)+pw_params%bandgap
            endif
            
            call diagonalize_hamiltonian(pw_params,
     $           pw_params%maxitdiag, idiaconv, bands%min,
     $           n_ukq,pw_params%epsdiag,
     $           e_kq(1), ham, u_kq(1,1))

c
c           setup nonlocal potentials and derivatives
c
            allocate(nloc_deriv_kq(k_gspace%length+pspot%nanl,
     $           pspot%nanl, 2,3))
            call setup_nonlocal_derivative(pw_params%output,qmag,
     $           kq(1),pspot, k_gspace, nloc_deriv_kq(1,1,1,1), crys)
c
c           if memory optimization is switched on, we have to recompute
c           the nonlocal derivative at k.

            if(iand(pw_params%optimize,1).eq.1) then
               allocate(nloc_deriv_k(k_gspace%length+pspot%nanl, 
     $              pspot%nanl,2,3)) 
               call setup_nonlocal_derivative(pw_params%output,qmag,
     $              rk(1),pspot,k_gspace,nloc_deriv_k(1,1,1,1), crys)
            endif

c
c           compute b0, p0 from q
c               
            allocate(u_kqtil(k_gspace%length,bands%ifmax(irk,1)))

            do iperm0 =1,-1,-2
               b0 = mod(iq+iperm0+2,3) +1 ! gives different b0 for iperm0
               p0 = mod(iq-iperm0+2,3) +1 ! gives p0 = abs(q x b0)
c
c              compute the gradient of uk
c
               allocate(graduk(k_gspace%length,bands%ifmax(irk,1)))
               do i=1, bands%ifmax(irk,1)
                  call take_nonloc_deriv(p0,k_gspace, rk(1),u_k(1,i),
     $                 ham%vnloc%nvecs,nloc_deriv_k(1,1,1,1),
     $                 graduk(1,i),crys) 
                  call take_nonloc_deriv(p0,ham%gspace, kq(1),
     $                 u_k(1,i),ham%vnloc%nvecs,nloc_deriv_kq(1,1,1,1)
     $                 ,phi(1), crys) 
                  
                  graduk(:,i)=0.5d0*(graduk(:,i)+phi(:))
               end do
c
c              compute u_kq~ for a given p0
c
               if(iand(pw_params%optimize,1).eq.1) then
                  deallocate(nloc_deriv_kq)
                  deallocate(nloc_deriv_k)
               endif

               if(iand(pw_params%optimize,4).eq.4) 
     $              u_kqtil(:,:)=u_ktil(:,:,p0)

               ham%shift=0.d0   ! no shift here
               call cg_blocksolve(pw_params%output,crys,p0,ham,
     $              pw_params%maxitcgsol,  pw_params%epsmag, 
     $              ksub_gspace, rk(1),
     $              bands%ifmax(irk,1),  ksub_gspace%length,
     $              bands%energy(1,irk,1), densham%energy(1),
     $              e_kq(1), denshamkq%energy(1), 
     $              denshamkq%matrix(1), 
     $              u_kq(1,1),
     $              u_k(1,1), 
     $              u_kqtil(1,1),iguess, graduk(1,1))
               deallocate(graduk)

c
c           re-setup nonlocal potentials and derivatives
c
               if(iand(pw_params%optimize,1).eq.1) then
                  allocate(nloc_deriv_kq(k_gspace%length+pspot%nanl,
     $                 pspot%nanl, 2,3))
                  call setup_nonlocal_derivative(pw_params%output
     $                 ,qmag,kq(1),pspot, k_gspace, nloc_deriv_kq(1,1
     $                 ,1,1), crys)
                  allocate(nloc_deriv_k(k_gspace%length+pspot%nanl,
     $                 pspot%nanl, 2,3))
                  call setup_nonlocal_derivative(pw_params%output
     $                 ,qmag,rk(1),pspot, k_gspace, nloc_deriv_k(1,1
     $                 ,1,1), crys)
               endif


               allocate(u_k_rspace(ffts%r_size))

               do i=1,bands%ifmax(irk,1) ! loop over occupied bands
c                 tsum1(iq)= tsum1(iq) + e_kq(i)*dble(iqsign)/(4.d0*qmag)
c
c                 Fourier transform u_(k,i)
c     
                  call fourier_transform(-1,ffts,k_gspace,u_k(1,i),
     $                 u_k_rspace(1))


                  do alf=1,3 
c
c                    apply operator p1 to u_k+q~
c                  
                     call take_nonloc_deriv(alf,k_gspace, kq(1), 
     $                    u_kqtil(1,i),0,nloc_deriv_kq(1,1,1,1),phi(1),
     $                    crys) 
c
c                    add to the "charge" density
c
                     call add_to_charge(ffts, k_gspace,
     $                    u_k_rspace(1), phi(1),
     $                    dble(iperm0)*dble(iqsign)*kpoints%w(irk),
     $                    rhom_r(1,b0,alf))


                  end do

                  allocate(u_kqtil_r(ffts%r_size))

                  call fourier_transform(-1,ffts,k_gspace,
     $                 u_kqtil(1,i), u_kqtil_r(1))

                  do alf=1,3 
c
c                    apply operator p1 to u_k
c                  
                     call take_nonloc_deriv(alf,k_gspace, rk(1),
     $                    u_k(1,i), 0,nloc_deriv_k(1,1,1,1),
     $                    phi(1), crys) 

c
c                    add to the "charge" density
c
                     call add_to_charge(ffts,k_gspace,u_kqtil_r(1), 
     $                    phi(1), -1.d0*dble(iperm0)* dble(iqsign)
     $                    *kpoints%w(irk), rhom_r(1,b0,alf)) 

                  end do

                  deallocate(u_kqtil_r)

c
c                 treat the G=0 component
c
                  do iperm1 =1,-1,-2
                     b1 = mod(iq+iperm1+2,3) +1 ! gives different b1 for iperm
                     p1 = mod(iq-iperm1+2,3) +1 ! gives p1 = abs(q x b1)

c
c                    --- half-half for G=0 ----
c
                     call take_nonloc_deriv(p1,k_gspace, rk(1),
     $                    u_kqtil(1,i),0,nloc_deriv_k(1,1,1,1),
     $                    phi(1),crys)  

                     call take_nonloc_deriv(p1,k_gspace, kq(1),
     $                    u_kqtil(1,i),0,nloc_deriv_kq(1,1,1,1),phi1(1),
     $                    crys)  
                     phi(:)=0.5d0*(phi(:)+phi1(:))


                     delchi= dble(iperm1)*dble(iperm0)*kpoints%w(irk)
     $                    *(parallel_zdotc2(k_gspace%length,
     $                    u_k(1,i),1,phi(1),1)-kk_termhh(p0,p1))
                     if(b0.eq.b1) then
                        chihh(b0,b1) = chihh(b0,b1) + delchi
                     else
                        chihh(b0,b1) = chihh(b0,b1) + 2.d0*delchi
                     endif
c
c                    --- dH/dk  dH/dk for G=0 ----
c
                     call take_nonloc_deriv(p1,k_gspace, rk(1),
     $                    u_kqtil(1,i),pspot%nanl,
     $                    nloc_deriv_k(1,1,1,1),phi(1),crys)  

                     call take_nonloc_deriv(p1,k_gspace, kq(1),
     $                    u_kqtil(1,i),pspot%nanl,
     $                    nloc_deriv_kq(1,1,1,1),phi1(1), crys)  
                     phi(:)=0.5d0*(phi(:)+phi1(:))


                     delchi= dble(iperm1)*dble(iperm0)*kpoints%w(irk)
     $                    *(parallel_zdotc2(k_gspace%length,
     $                    u_k(1,i),1,phi(1),1)-kk_termvv(p0,p1))

                     if(b0.eq.b1) then
                        chivv(b0,b1) = chivv(b0,b1) + delchi
                     else
                        chivv(b0,b1) = chivv(b0,b1) + 2.d0*delchi
                     endif

                  end do        ! loop over permutation iperm1


               end do           ! end of loop over bands i
               deallocate(u_k_rspace)    
               call myflush(9)
            end do              ! loop over permutation iperm0
            deallocate(u_kqtil)
            deallocate(nloc_deriv_kq)
            if(iand(pw_params%optimize,1).eq.1) then
               deallocate(nloc_deriv_k)              
            endif
         end do                 ! end of iqsign
         end do                 ! end of loop over q-points
         
         deallocate(phi)
         deallocate(phi1)
         if(iand(pw_params%optimize,1).ne.1) then
            deallocate(nloc_deriv_k)              
         endif
         if(iand(pw_params%optimize,4).eq.4) deallocate(u_ktil)
        

         call destroy(densham)
         call destroy(denshamkq)
         call destroy(ksub_gspace)
         call destroy(k_gspace)
         deallocate(u_k)
         deallocate(u_kq)
         call destroy(ham)

c         write(9,*) tsum
c         write(9,*) 'tsum1:'
c         write(9,*) tsum1
c         write(9,*) 'tsum1-tsum:'
c         write(9,*) tsum1-tsum

         write(9,926) gimmetime()-tk
c
c        do checkpointing if required
c
         if(iand(pw_params%miscflag,2).eq.2) 
     $        call chkpt_chi(irk,rhom_r(1,1,1),9*pot_gspace%r_size,
     $        pot_gspace%myproc, chivv,chihh, pw_params%output,1)

      end do                    ! end of loop over kpoints

      deallocate(vloc)
      deallocate(e_kq)

c
c          spin    average q     Hartree 
c
      chivv=-2.d0 *   0.5d0    *   2.d0 * dble(chivv(:,:))/(qmag
     $     *qmag) 
c
c     first chi with v-v
c
      chi_macro=dble(chivv)*(.529177d-8)**3.d0 * 0.6022d24 / (137.036)
     $     **2.d0* 1.d6 

      call  symmetrize_tensor(chi_macro(1,1),   crys, syms)

      write(9,190);write(9,200); call printdmatrix(chi_macro(1,1),3,3)

      chi_avg = (chi_macro(1,1)+chi_macro(2,2)+chi_macro(3,3))/3.d0 

      write(9,400) chi_avg

      chi_avg = (chi_macro(1,1)*g0mask(1,1)
     $     +chi_macro(2,2)*g0mask(2,2)+chi_macro(3,3)*g0mask(3,3))/3.d0 

      write(9,410) chi_avg/(crys%vcell*(.529177d-8)**3.d0 
     $     * 0.6022d24)*2*pi2

c
c     now chi computed with half-half
c
      chihh=-2.d0*0.5d0*2.d0*dble(chihh(:,:))/(qmag*qmag) 
      chi_macro=dble(chihh)*(.529177d-8)**3.d0 * 0.6022d24 / (137.036)
     $     **2.d0* 1.d6 

      call  symmetrize_tensor(chi_macro(1,1),   crys, syms)

      write(9,210);   call printdmatrix(chi_macro(1,1),3,3)

      chi_avg = (chi_macro(1,1)+chi_macro(2,2)+chi_macro(3,3))/3.d0 

      write(9,400) chi_avg

      chi_avg = (chi_macro(1,1)*g0mask(1,1)
     $     +chi_macro(2,2)*g0mask(2,2)+chi_macro(3,3)*g0mask(3,3))/3.d0 

      write(9,415) chi_avg/(crys%vcell*(.529177d-8)**3.d0 
     $     * 0.6022d24)*2*pi2
c
c     compute the  magnetic susceptibilty
c
      write(9,350) (g0mask(i,1:3),i=1,3)
      call wrapup_chi(crys,ffts, pot_gspace, rhom_r(1,1,1),
     $     qmag,g0mask(1,1),chi_g(1,1,1),chihh(1,1))

      call symmetrize_tensor_global(pw_params%output, 
     $     chi_g(1,1,1),syms,pot_gspace)

      call writegsdat(1, pot_gspace, chi_g(1,1,1), 9,9,1,'CHI',3)
 

      call nmr_shift(pw_params,crys,pot_gspace,chi_g(1,1,1))

      write(9,420) qmag
      write(9,300)

      deallocate(rhom_r)

      return

 150  format(/' *** WARNING: ACCURACY OF DIAGONALIZATION ',
     $     'MIGHT NOT BE HIGH ENOUGH:',g12.6)

 190  format(/' *** WARNING: CHI IS ONLY CORRECT IF SYMMETRY-',
     $       /' *** OPERATIONS MAP CARTESIAN AXIS INTO EACH OTHER')

 200  format(/' MACROSCOPIC CHI (nonlocal velocity operator)',
     $     /' [10^-6 cm^3/mole]:'/)
 210  format(/' MACROSCOPIC CHI (half-half)',
     $     /' [10^-6 cm^3/mole]:'/)
 220  format(/' G=0 CORRECTION TO THE NMR SHIFT',
     $     '(nonlocal velocity operator):')
 222  format(/' G=0 CORRECTION TO THE NMR SHIFT',
     $     '(half-half):')
 300  format(/' THE NMR SHIFT IS COMPUTED WITH HALF/HALF AND',
     $     ' THE G=0 ALSO WITH HH')
 350  format(/' THE G=0 COMPONENT FOR THE SHIFT IS MASKED WITH:'/,
     $     3(3f12.6/))
 400  format(/' AVERAGE CHI:', f12.4,' *10^-6 cm^3/mole')

 410  format(/' VV SHIFT CONTRIBUTION:', f12.4)
 415  format(/' HH SHIFT CONTRIBUTION:', f12.4)
 420  format(/' NMR Q:', f10.6)

 800  format(//' COMPUTING CONTRIBUTION FROM KPOINT',i4,':',3f9.4)

 910  format(' TIME FOR DIAGONALIZATION:', f12.3)
 922  format(' TIME FOR NONLOCAL POTENTIAL:', f12.3)
 926  format(' TIME FOR KPOINT:', f12.3)
 930  format(' TIME [SECONDS] FOR ADD_CHARGE:',f12.3)
      end

