c     -*-Fortran-*-
c
      subroutine pw91_excorr(ipr, ffts, crys, gs,  inputcd,
     $     exc, afmagm)

      use all_to_all_module
      include 'use.h'
      implicit none
      include 'interface.h'
      include 'all_to_all.h'

c
c     INPUT:
c     ------

      type (fftstruc) ffts      ! for the fourier transform
      type (crystal) crys       ! for the bvec array to take the gradient
      type (parallel_gspace) gs ! the gspace for the gradient computation


      integer 
     $     ipr                  ! printout flag

c
c     INPUT/OUTPUT:
c     ------------
c
c     the charge density in fourier space on input, the exchange
c     -correlation potential (Rydbergs) in fourier space on output
c
      complex(kind=8) 
     $     inputcd(gs%length,crys%nspin) 
c
c
c     OUTPUT:
c     ------
c
      double precision 
     $     afmagm,              ! the antiferromagnetic moment in mu_Bohr
     $     exc                  ! the exchange-correlation energy in Rydbergs

c
c     DESCRIPTION:
c     -----------
c       
c     Computes the exchange correlation potential of Perdew-Wang 91
c     as given in:
c     Y.-M. Juan and E. Kaxiras, PRB 48, 14944 (1993),
c     (in Rydberg) given the charge density (in electrons per unit cell).
c     The potential replaces the density in the array inputcd.
c     This supports spin polarization.
c      
c     The implementation follows:
c     J.A. White and D.M. Bird PRB 50, 4954 (1994).
c     i.e.:
c
c     Exc=V_cell/N_r Sum_r F_xc(n(r),|grad n(r)|)                          (1)
c
c     Vxc(r)= dF_xc/dn(r) -  1/N_r Sum_{r',G} dF_xc/d|grad n(r')| 
c             iG*grad n(r')/|grad n(r')|*exp[iG*(r-r')]                    (2)
c
c     To evaluate eq. (2) 8 FFTs are involved.
c
c
c
c     1998 by Young-Gui Yoon. Based on an earlier version by Francesco 
c     Mauri, Bernd Pfrommer, and Alfredo Pasquarello.
c

c     --------------------- local variables -------------------------

      double precision 
     $ bb1,bb2,bb3,bb4,bb5,alfa,beta,eta,cc0,cc1,c1,c2,c3,c4,c5,c6,c7,
     $ a,alfa1,bt1,bt2,bt3,bt4,a_p,alfa1_p,bt1_p,bt2_p,bt3_p,bt4_p,
     $ a_alf,alfa1_alf,bt1_alf,bt2_alf,bt3_alf,bt4_alf,alfr,alfden,
     $ alflog,dalfdrs,zeta,zeta3,zeta4,fzeta,fzetap,fzetazeta4,
     $ invfzz,gam,invgam,phi,phi2,phi3,phi4,invphi,invphi3,phip,
     $ pi,pi2,delt,x13,x23,x43,x76,x16m,ax,pider1,pider2,pider3,
     $ abder1,abder2,abder3,vcell,invcell,spscale,invspscale,
     $ roe,grx,gry,grz,roeth,rs,gkf,sd,s,s2,s3,s4,
     $ ysr,ys,ysl,sysl,fxexp,fxnum,fxden,fx,rs12,rs32,rs2,rs3,invrs12,
     $ td,t,t2,t3,t4,aexp,abig,abig2,h0num,h0den,h0arg,h0,
     $ ccrnum,ccrden,ccr,h1,dys,dfs,decd,ecr_dif,
     $ dh0da,dadec,dadz,decdz,dh0d,dh0dt,dh0dg,dh0dz,
     $ dcdrs,dh1drs,dh1d,dh1dt,dh1ds,dh1dg,dh1dz,
     $ byagr,dexcdg,normfac
C
C     _________________________________________________________________
      integer 
     $ i,k,ir,nspin,rsize,is,isign,norm

      double precision, dimension(1:3) :: aroe,agr,ecr,decdrs
      double precision, dimension(1:2) :: xchge,ecden,eclog,
     $                                    dfxd,dfxdg,dexcd
      double precision, allocatable ::
     $     rhoe(:,:),         ! charge density in rspace
     $     gradr(:,:,:)       ! gradient of charge density in rspace

      complex(kind=8), allocatable :: 
     $     gswork(:),         ! work array in gspace
     $     gswork1(:),        ! work array in gspace
     $     gswork2(:),        ! work array in gspace
     $     rswork(:)          ! work array in rspace
c
c     -------------------------------------------------------------------
c
c     pw91 parameters
c
c     The following three parameters are overwritten by exact expression
c     Reference: J. Perdew et. al., PRB 46, 6671 (1992)
c     
c     beta=(16/pi)*[(3*pi**2)**(1/3)]*0.004235 instead of 0.0667263212
c     cc0=(16/pi)*[(3*pi**2)**(1/3)] instead of 15.75592
c     cc1=0.004235-(3/7)*0.001667 instead of 0.003521
c
      parameter(pi=3.14159265358979323846264338327950d0)
      parameter(ax=-0.738558766382022405884230032680836d0)
      parameter(gam=0.5198420997897463295344212145565d0)
      parameter(invgam=1.d0/gam,invfzz=9.d0*gam/8.d0)
      parameter(eta=1.d-12,delt=1.d-12)
      parameter(bb1=0.19645,bb2=0.27430,bb3=-0.15084,bb4=0.004,
     1 bb5=7.7956,alfa=0.09,beta=0.06672632268006,cc0=15.75592034948314,
     2 cc1=0.00352057142857,c1=0.001667,c2=0.002568,c3=0.023266,
     3 c4=7.389e-6,c5=8.723,c6=0.472,c7=7.389e-2,a=0.0621814,
     4 alfa1=0.2137,bt1=7.5957,bt2=3.5876,bt3=1.6382,bt4=0.49294,
     5 a_p=0.0310907,alfa1_p=0.20548,bt1_p=14.1189,bt2_p=6.1977,
     6 bt3_p=3.3662,bt4_p=0.62517,a_alf=0.033774,alfa1_alf=0.11125,
     7 bt1_alf=10.357,bt2_alf=3.6231,bt3_alf=0.88026,bt4_alf=0.49671) 
c
      logical isp               ! spin polarization
c
c     -------------------------------------------------------------------
c
      x13=1.d0/3.0d0
      x16m=-1.d0/6.0d0
      x23=2.0d0/3.0d0
      x43=4.0d0/3.0d0
      x76=7.0d0/6.0d0
c     _________________________________________________________________
c     derived parameters from pi
c
      pi2=pi*pi
      pider1=(0.75d0/pi)**x13
      pider2=(3.0d0*pi2)**x13
      pider3=(3.0d0*pi2/16.0d0)**x13
c     _________________________________________________________________
c     derived parameters from alfa and beta 
c
      abder1=beta*beta/(2.0d0*alfa)
      abder2=1.d0/abder1
      abder3=2.0d0*alfa/beta
c
      norm  = gs%fftsize(1)*gs%fftsize(2)*gs%fftsize(3)
      normfac=crys%vcell/norm
      vcell=crys%vcell
      invcell=1.d0/crys%vcell
c
      nspin=crys%nspin
      spscale=dble(nspin)
      invspscale=1.d0/spscale
      isp = .false.
      if(nspin.eq.2) isp =.true.
c
      rsize=gs%r_size
      exc=0.d0
c     _________________________________________________________________
c     compute charge density in real space (rhoe) and gradient of the
c     charge density in real space (gradr).
c     rhoe(:,is) grad(:,:,is) coresponds to spin is; note that
c     gradr(:,:,3) corresponds to gradient of total charge.
c
      phi=1.d0
      invphi=1.d0
      phi2=1.d0
      phi3=1.d0
      invphi3=1.d0
      phi4=1.d0
c
      allocate(rswork(rsize))
      allocate(gswork(gs%length))
      allocate(gswork1(gs%length))
      allocate(gswork2(gs%length))
c
      allocate(rhoe(rsize,nspin))
      allocate(gradr(rsize,3,2*nspin-1))
c
      do is=1,nspin
         gswork(:) = inputcd(:,is) * invcell
         call fourier_transform(-1,ffts,gs,gswork(1),rswork(1))
         rhoe(:,is) = real(rswork(:))

   
         do k=1,3
            call derivative_gspace(k,inputcd(1,is),gs,crys,gswork(1))
            gswork(:) = gswork(:) * invcell * (0.d0,1.d0)
            call fourier_transform(-1,ffts,gs,gswork(1),rswork(1))
            gradr(:,k,is) = real(rswork(:))
         enddo
      enddo
c     _________________________________________________________________
c     compute antiferromagnetic moment
c
      afmagm=0.d0

      if(isp) then              ! calculate antiferromagnetic moment
         do i=1,rsize
            afmagm=afmagm+abs(rhoe(i,1)-rhoe(i,2))
         end do
         call all_sum_all(afmagm)
         if(norm.gt.0) then
            afmagm=vcell*afmagm/dble(norm) ! normalize
         else
            print *,'pw91_excorr: norm=0. Something is wrong.'
            stop
         endif
c
         gradr(:,:,3)=gradr(:,:,1)+gradr(:,:,2)
c
      endif
c
c     _________________________________________________________________
c     main loop
c
      do 100 ir=1,rsize
         do is=1,nspin
            roe=rhoe(ir,is)*spscale
            if((.not.isp).and.(roe.le.0.d0)) then
               gradr(ir,:,1)=0.d0
               goto 100
            endif
            if(isp.and.(roe.lt.0.d0)) then
               gradr(ir,:,:)=0.d0
               goto 100
            endif
            aroe(is)=abs(roe)
            grx=gradr(ir,1,is)*spscale
            gry=gradr(ir,2,is)*spscale
            grz=gradr(ir,3,is)*spscale
            agr(is)=sqrt(grx*grx+gry*gry+grz*grz)
            roeth=aroe(is)**x13 
            if(isp.and.(roe.eq.0.d0)) then
               xchge(is)=0.d0
               dfxd(is)=0.d0
               dfxdg(is)=0.d0
            else
               rs= pider1/roeth
               gkf=pider2*roeth
               sd=1.d0/(2.0d0*gkf*aroe(is))
               s=agr(is)*sd
               s2=s*s
               s3=s*s2
               s4=s2*s2
c     _________________________________________________________________
c     exchange 
c
               ysr=sqrt(1.d0+bb5*bb5*s2)
               ys=bb5*s+ysr
               ysl=log(ys)*bb1
               sysl=s*ysl
               fxexp=exp(-100.0d0*s2)
               fxnum=1.d0+sysl+(bb2+bb3*fxexp)*s2
               fxden=1.d0/(1.d0+sysl+bb4*s4)
               fx=fxnum*fxden
               xchge(is)=ax*fx*roeth
c     _________________________________________________________________
c     first part xc-potential from exchange  
c
               dys=bb5*(1.d0+bb5*s/ysr)/ys
               dfs=-fxnum*(ysl+bb1*s*dys+4.0d0*bb4*s3)*fxden*fxden
     $              +(ysl+bb1*s*dys+2.0d0*s*(bb2+bb3*fxexp)
     $              -200.0d0*s3*bb3*fxexp)*fxden
               dfxd(is)=(ax*roeth*x43)*(fx-dfs*s)
               dfxdg(is)=ax*roeth*dfs*sd
            endif
         enddo
c     _________________________________________________________________
c     back to total charge density  
c
         if(isp) then
            roe=rhoe(ir,1)+rhoe(ir,2)
            if(roe.eq.0.d0) then
               rhoe(ir,:)=0.d0
               gradr(ir,:,:)=0.d0
               goto 100
            endif
            aroe(3)=abs(roe)
            grx=gradr(ir,1,3)
            gry=gradr(ir,2,3)
            grz=gradr(ir,3,3)
            agr(3)=sqrt(grx*grx+gry*gry+grz*grz)
            roeth=aroe(3)**x13
            rs= pider1/roeth
            gkf=pider2*roeth
            sd=1.d0/(2.0d0*gkf*aroe(3))
            s=agr(3)*sd
            s2=s*s
          endif
c     _________________________________________________________________
c     correlation ecr=ec(rho,zeta) 
c
         rs12=sqrt(rs)
         invrs12=1.d0/rs12
         rs32=rs12*rs
         rs2=rs*rs
         rs3=rs*rs2
         ecden(1)=a*(bt1*rs12+bt2*rs+bt3*rs32+bt4*rs2) 
         eclog(1)=log(1.d0+(1.d0/ecden(1)))
         ecr(1)=-a*(1.d0+alfa1*rs)*eclog(1)
         if(isp) then
            ecden(2)=a_p*(bt1_p*rs12+bt2_p*rs+bt3_p*rs32+bt4_p*rs2) 
            eclog(2)=log(1.d0+(1.d0/ecden(2)))
            ecr(2)=-a_p*(1.d0+alfa1_p*rs)*eclog(2)
c
            alfden=a_alf*(bt1_alf*rs12+bt2_alf*rs
     $             +bt3_alf*rs32+bt4_alf*rs2) 
            alflog=log(1.d0+(1.d0/alfden))
            alfr=a_alf*(1.d0+alfa1_alf*rs)*alflog
c
            zeta=(rhoe(ir,1)-rhoe(ir,2))/roe
            if(abs(zeta).gt.1.d0) zeta=dble(sign(1.d0,zeta))
            zeta3=zeta*zeta*zeta
            zeta4=zeta3*zeta
c           invgam=1.0/(2.d0**(3.d0/4.d0)-2.d0)
            fzeta=((1.d0+zeta)**x43+(1.d0-zeta)**x43-2.d0)*invgam
            fzetap=x43*((1.d0+zeta)**x13-(1.d0-zeta)**x13)*invgam
            fzetazeta4=fzeta*zeta4
            phi=((1.d0+zeta)**x23+(1.d0-zeta)**x23)*0.5d0
            phip=(((1.d0+zeta)**2+eta)**x16m 
     $           -((1.d0-zeta)**2+eta)**x16m)*x13
            invphi=1.d0/phi
            phi2=phi*phi
            phi3=phi2*phi
            invphi3=1.d0/phi3
            phi4=phi2*phi2
c
            ecr_dif=ecr(2)-ecr(1)
            ecr(3)=ecr(1)+alfr*fzeta*invfzz*(1.d0-zeta4)
     $          +ecr_dif*fzetazeta4
            decdz=4.d0*zeta3*fzeta*(ecr_dif-alfr*invfzz)
     $          +fzetap*(zeta4*ecr_dif+(1.d0-zeta4)*alfr*invfzz)
         endif
c     _________________________________________________________________
c     correlation h0(t,ecr,zeta)
c
         td=pider3*sd*invrs12*invphi
         t=agr(2*nspin-1)*td
         t2=t*t
         t3=t*t2
         t4=t2*t2
         aexp=exp(-abder2*ecr(2*nspin-1)*invphi3)-1.d0
         abig=abder3/aexp
         abig2=abig*abig
         h0num=t2+abig*t4
         h0den=1.d0/(1.d0+abig*t2+abig2*t4)
         h0arg=1.d0+abder3*h0num*h0den
         h0=abder1*log(h0arg)*phi3
c     _________________________________________________________________
c     correlation h1(t,s,aroe,zeta)
c
         ccrnum=c2+c3*rs+c4*rs2
         ccrden=1.d0/(1.d0+c5*rs+c6*rs2+c7*rs3)
         ccr=c1+ccrnum*ccrden
         fxexp=exp(-100.0d0*s2*phi2)
         h1=phi3*cc0*(ccr-cc1)*t2*fxexp
c     _________________________________________________________________
c     updating of xc-energy
c
         exc=exc+(ecr(2*nspin-1)+h0+h1)*aroe(2*nspin-1)
         do is=1,nspin
            exc=exc+xchge(is)*aroe(is)*invspscale
         enddo    
c     _________________________________________________________________
c     first part xc-potential from ecr 
c
         decdrs(1)=-a*alfa1*eclog(1)*rs+a*(1.d0+alfa1*rs)
     $        *a*(0.5d0*bt1*rs12+bt2*rs+1.5d0*bt3*rs32+2.0d0*bt4*rs2)
     $        /(ecden(1)*ecden(1)+ecden(1))
         if(isp) then
            decdrs(2)=-a_p*alfa1_p*eclog(2)*rs+a_p*(1.d0+alfa1_p*rs)
     $          *a_p*(0.5d0*bt1_p*rs12+bt2_p*rs+1.5d0*bt3_p*rs32
     $          +2.0d0*bt4_p*rs2)/(ecden(2)*ecden(2)+ecden(2))
            dalfdrs=a_alf*alfa1_alf*alflog*rs-a_alf*(1.d0+alfa1_alf*rs)
     $          *a_alf*(0.5d0*bt1_alf*rs12+bt2_alf*rs+1.5d0*bt3_alf*rs32
     $          +2.0d0*bt4_alf*rs2)/(alfden*alfden+alfden)
            decdrs(3)=decdrs(1)*(1-fzetazeta4)+decdrs(2)*fzetazeta4
     $          +dalfdrs*fzeta*invfzz*(1-zeta4)
         endif
         decd=-x13*decdrs(2*nspin-1)
c     _________________________________________________________________
c     first part xc-potential from h0 
c
         dh0da=phi3*abder1/h0arg*abder3*h0den*(t4-h0num*h0den*(t2+2.0d0
     $        *abig*t4))
         dadec=abder3*abder2*(aexp+1.d0)/(aexp*aexp)*invphi3
         if(isp) dadz=dadec*(decdz-ecr(2*nspin-1)*3.d0*invphi*phip)
         dh0d=dh0da*dadec*decd 
         dh0dt=phi3*abder1/h0arg*abder3*h0den
     $        *(2.0d0*t+4.0d0*abig*t3-h0num*h0den*(2.0d0*abig*t+4.0d0
     $        *abig2*t3))
         dh0d=dh0d-x76*t*dh0dt
         dh0dg=dh0dt*td
         if(isp) dh0dz=dh0da*dadz+(3.d0*h0-dh0dt*t)*phip*invphi
c     _________________________________________________________________
c     first part xc-potential from h1 
c
         dcdrs=(c3+2.0d0*c4*rs-ccrnum*ccrden*(c5+2.0d0*c6*rs+3.0d0*c7
     $        *rs2))*ccrden
         dh1drs=phi3*cc0*t2*fxexp*dcdrs
         dh1d=-x13*rs*dh1drs
         dh1dt=phi3*2.0d0*t*cc0*(ccr-cc1)*fxexp
         dh1d=dh1d-x76*t*dh1dt
         dh1ds=-200.0d0*phi2*s*cc0*(ccr-cc1)*phi3*t2*fxexp
         dh1d=dh1d-x43*s*dh1ds
         dh1dg=dh1dt*td+dh1ds*sd
         if(isp) dh1dz=(3.d0*phip*invphi-200.d0*phi*phip*s2)*h1
c     _________________________________________________________________
c     first part xc-potential 
c
      do is=1,nspin
         dexcd(is)=dfxd(is)+decd+dh0d+dh1d+ecr(2*nspin-1)+h0+h1
      enddo
         isign=sign(1.d0,agr(2*nspin-1)-delt)
         byagr=0.5d0*(1+isign)/(agr(2*nspin-1)+(1-isign)*delt)
c
      if(isp) then
         dexcdg=(dh0dg+dh1dg)*aroe(2*nspin-1)*byagr
         do is=1,nspin
            rhoe(ir,is)=dexcd(is) 
     $                -(zeta-2.d0*(1.5d0-is))*(decdz+dh0dz+dh1dz)
            isign=sign(1.d0,agr(is)-delt)
            byagr=0.5d0*(1+isign)/(agr(is)+(1-isign)*delt)
            gradr(ir,:,is)=spscale*gradr(ir,:,is)*dfxdg(is)
     $                    *aroe(is)*byagr
         enddo
      else
         rhoe(ir,1)=dexcd(1)
         dexcdg=(dfxdg(1)+dh0dg+dh1dg)*aroe(2*nspin-1)*byagr
      endif
c
         gradr(ir,:,2*nspin-1)=gradr(ir,:,2*nspin-1)*dexcdg
 100  continue
c     _________________________________________________________________
c     sum exc over all processors
      call all_sum_all(exc)
c     _________________________________________________________________
c     au to ry
c
      exc=exc*2.d0*normfac
c     _________________________________________________________________
c     second part xc-potential: 3 forward ffts  
c
c     gradr(:,k) now contains df_xc/d|grad n(r')| grad n(r')/|grad n(r')| 
c
      do is=1,nspin
         gswork2(:)=0.d0
         do k=1,3
            if(isp) then
               rswork(:)=gradr(:,k,3)+gradr(:,k,is)
            else
               rswork(:)=gradr(:,k,1)
            endif
            call fourier_transform(1,ffts,gs,gswork(1),rswork(1))
            call derivative_gspace(k,gswork(1),gs,crys,gswork1(1))
            gswork2(:)=gswork2(:)+gswork1(:) * (0.d0,1.d0)
         enddo
c
c     _________________________________________________________________
c     second part xc-potential: 1 inverse fft  
c
         call fourier_transform(-1,ffts,gs,gswork2(1),rswork(1))
         rswork(:)=rhoe(:,is)-rswork(:)
c
c     _________________________________________________________________
c
         call fourier_transform(1,ffts,gs,inputcd(1,is),rswork(1))
      enddo
c
c     _________________________________________________________________
c     au to ry
c
      inputcd(:,:)=inputcd(:,:)*2.d0
C     _________________________________________________________________
      deallocate(rswork)
      deallocate(gswork)
      deallocate(gswork1)
      deallocate(gswork2)
c
      deallocate(rhoe)
      deallocate(gradr)
C     _________________________________________________________________
      return
      end
c
c
c
C     _________________________________________________________________
c     -*-Fortran-*-
c
      subroutine pbe_excorr(ipr, ffts, crys, gs,  inputcd,
     $     exc, afmagm)

      use all_to_all_module
      include 'use.h'
      implicit none
      include 'interface.h'
      include 'all_to_all.h'

c
c     INPUT:
c     ------

      type (fftstruc) ffts      ! for the fourier transform
      type (crystal) crys       ! for the bvec array to take the gradient
      type (parallel_gspace) gs ! the gspace for the gradient computation


      integer 
     $     ipr                  ! printout flag

c
c     INPUT/OUTPUT:
c     ------------
c
c     the charge density in fourier space on input, the exchange
c     -correlation potential (Rydbergs) in fourier space on output
c
      complex(kind=8) 
     $     inputcd(gs%length,crys%nspin) 
c
c
c     OUTPUT:
c     ------
c
      double precision 
     $     afmagm,              ! the antiferromagnetic moment in mu_Bohr
     $     exc                  ! the exchange-correlation energy in Rydbergs

c
c     DESCRIPTION:
c     -----------
c       
c     Computes the exchange correlation potential of  PBE  as given in 
c     J. Perder, K. Burke, and M. Ernzerhorf, PRL 77, 3865 (1996) in 
c     Rydberg  given  the  charge  density  in electrons per unit cell.
c     The potential  replaces the density in the array inputcd. This 
c     supports spin polarization.
c      
c     The implementation follows:
c     J.A. White and D.M. Bird PRB 50, 4954 (1994).
c     i.e.:
c
c     Exc=V_cell/N_r Sum_r F_xc(n(r),|grad n(r)|)                          (1)
c
c     Vxc(r)= dF_xc/dn(r) -  1/N_r Sum_{r',G} dF_xc/d|grad n(r')| 
c             iG*grad n(r')/|grad n(r')|*exp[iG*(r-r')]                    (2)
c
c     To evaluate eq. (2) 8 FFTs are involved.
c
c
c
c     1998 by Young-Gui Yoon. Based on an earlier version of pw91_excorr.f
c

c     --------------------- local variables -------------------------

      double precision 
     $ bet,gamma,gam,invgam,um,uk,uk2,eta,delt,
     $ a,alfa1,bt1,bt2,bt3,bt4,a_p,alfa1_p,bt1_p,bt2_p,bt3_p,
     $ bt4_p,a_alf,alfa1_alf,bt1_alf,bt2_alf,bt3_alf,bt4_alf,
     $ alfr,alfden,alflog,dalfdrs,ecr_dif,
     $ zeta,zeta3,zeta4,fzeta,fzetap,fzetazeta4,invfzz,
     $ phi,phi3,invphi,invphi3,phip,
     $ x13,x23,x43,x76,x16m,pi,pi2,ax,pider1,pider2,pider3,
     $ abder1,abder2,abder3,vcell,invcell,spscale,invspscale,
     $ roe,grx,gry,grz,roeth,rs,gkf,sd,s,s2,fxden,fx,
     $ rs12,rs32,rs2,rs3,dfs,decd,dexcdg,byagr,
     $ td,t,t2,t3,t4,aexp,abig,abig2,hnum,hden,harg,h,
     $ dhda,dadec,dadz,decdz,dhd,dhdt,dhdg,dhdz,normfac
C
C     _________________________________________________________________
      integer 
     $ k,ir,nspin,rsize,is,isign,norm

      double precision, dimension(1:3) :: aroe,agr,ecr,decdrs
      double precision, dimension(1:2) :: xchge,ecden,eclog,
     $                                    dfxd,dfxdg,dexcd
      double precision, allocatable ::
     $     rhoe(:,:),         ! charge density in rspace
     $     gradr(:,:,:)       ! gradient of charge density in rspace

      complex(kind=8), allocatable :: 
     $     gswork(:),         ! work array in gspace
     $     gswork1(:),        ! work array in gspace
     $     gswork2(:),        ! work array in gspace
     $     rswork(:)          ! work array in rspace
c
c     -------------------------------------------------------------------
c
c     pbe parameters
c
c     -------------------------------------------------------------------
c     Formulas:
c        gam= 2^(4/3)-2
c        fzz=f''(0)= 8/(9*gam)
c        gamma=(1-log(2))/pi^2
c        bet=coefficient in gradient expansion for correlation.
c        eta=small number to stop d phi/ dzeta from blowing up.
c        e_x[unif]=ax*rho^(4/3)  [LDA]
c        ax = -0.75*(3/pi)^(1/3)
c        e_x[PBE]=e_x[unif]*FxPBE(s)
c        FxPBE(s)=1+uk-uk/(1+(um/uk)*s*s)            
c     -------------------------------------------------------------------
      parameter(pi=3.14159265358979323846264338327950d0)
      parameter(ax=-0.738558766382022405884230032680836d0)
      parameter(um=0.2195149727645171d0,uk=0.804d0)
      parameter(gam=0.5198420997897463295344212145565d0)
      parameter(invgam=1.d0/gam,invfzz=9.d0*gam/8.d0)
      parameter(gamma=0.03109069086965489503494086371273d0)
      parameter(bet=0.06672455060314922d0)
      parameter(eta=1.d-12,delt=1.d-12)    
      parameter(a=0.0621814,alfa1=0.2137,bt1=7.5957,bt2=3.5876,
     1 bt3=1.6382,bt4=0.49294,a_p=0.0310907,alfa1_p=0.20548,
     2 bt1_p=14.1189,bt2_p=6.1977,bt3_p=3.3662,bt4_p=0.62517,
     3 a_alf=0.033774,alfa1_alf=0.11125,bt1_alf=10.357,
     4 bt2_alf=3.6231,bt3_alf=0.88026,bt4_alf=0.49671) 
c
      logical isp               ! spin polarization
c
c     -------------------------------------------------------------------
c
      x13=1.d0/3.0d0
      x16m=-1.d0/6.0d0
      x23=2.0d0/3.0d0
      x43=4.0d0/3.0d0
      x76=7.0d0/6.0d0
c     _________________________________________________________________
c     derived parameters from pi
c
      pi2=pi*pi
      pider1=(0.75d0/pi)**x13
      pider2=(3.0d0*pi2)**x13
      pider3=(3.0d0*pi2/16.0d0)**x13
c     _________________________________________________________________
c     derived parameters from beta, gamma, and kappa
c
      abder1=gamma
      abder2=1.d0/abder1
      abder3=bet*abder2
      uk2=uk*uk
c
      norm  = gs%fftsize(1)*gs%fftsize(2)*gs%fftsize(3)
      normfac=crys%vcell/norm
      vcell=crys%vcell
      invcell=1.d0/crys%vcell
c
      nspin=crys%nspin
      spscale=dble(nspin)
      invspscale=1.d0/spscale
      isp = .false.
      if(nspin.eq.2) isp =.true.
c
      rsize=gs%r_size
      exc=0.d0
c     _________________________________________________________________
c     compute charge density in real space (rhoe) and gradient of the 
c     charge density in real space (gradr).
c     rhoe(:,is) grad(:,:,is) coresponds to spin is; note that 
c     gradr(:,:,3) corresponds to gradient of total charge.
c
      phi=1.d0
      invphi=1.d0
      phi3=1.d0
      invphi3=1.d0
c
      allocate(rswork(rsize))
      allocate(gswork(gs%length))
      allocate(gswork1(gs%length))
      allocate(gswork2(gs%length))
c
      allocate(rhoe(rsize,nspin))
      allocate(gradr(rsize,3,2*nspin-1))
c
      do is=1,nspin
         gswork(:) = inputcd(:,is) * invcell
         call fourier_transform(-1,ffts,gs,gswork(1),rswork(1))
         rhoe(:,is) = real(rswork(:))

   
         do k=1,3
            call derivative_gspace(k,inputcd(1,is),gs,crys,gswork(1))
            gswork(:) = gswork(:) * invcell * (0.d0,1.d0)
            call fourier_transform(-1,ffts,gs,gswork(1),rswork(1))
            gradr(:,k,is) = real(rswork(:))
         enddo
      enddo
c     _________________________________________________________________
c     compute antiferromagnetic moment and gradient of total charge
c
      afmagm=0.d0

      if(isp) then              ! calculate antiferromagnetic moment
         do ir=1,rsize
            afmagm=afmagm+abs(rhoe(ir,1)-rhoe(ir,2))
         end do
         call all_sum_all(afmagm)
         if(norm.gt.0) then
            afmagm=vcell*afmagm/dble(norm) ! normalize
         else
            print *,'pw91_excorr: norm=0. Something is wrong.'
            stop
         endif
c
         gradr(:,:,3)=gradr(:,:,1)+gradr(:,:,2)
c
      endif
c
c     _________________________________________________________________
c     main loop
c
      do 100 ir=1,rsize
         do is=1,nspin
            roe=rhoe(ir,is)*spscale        ! spin-scaled for exchange
            if((.not.isp).and.(roe.le.0.d0)) then
               gradr(ir,:,1)=0.d0
               goto 100
            endif
            if(isp.and.(roe.lt.0.d0)) then
               gradr(ir,:,:)=0.d0
               goto 100
            endif
            aroe(is)=abs(roe)
            grx=gradr(ir,1,is)*spscale
            gry=gradr(ir,2,is)*spscale
            grz=gradr(ir,3,is)*spscale
            agr(is)=sqrt(grx*grx+gry*gry+grz*grz)
            roeth=aroe(is)**x13 
            if(isp.and.(roe.eq.0.d0)) then
               xchge(is)=0.d0
               dfxd(is)=0.d0
               dfxdg(is)=0.d0
            else
               rs= pider1/roeth
               gkf=pider2*roeth
               sd=1.d0/(2.0d0*gkf*aroe(is))
               s=agr(is)*sd
               s2=s*s
c     _________________________________________________________________
c     exchange and first part xc-potential from exchange
c           d[]d means (d[]/dn) for dfxd
c           d[]d means n(d[]/dn) otherwise
c           d[]dg means d[]/d|grad(n)|/n for dfxdg
c           d[]dg means d[]/d|grad(n)| otherwise
c           Be careful! fx for dfxd and dfxdg gives exchange energy
c           when integrated; others are exchange enhancement factor.
c
               fxden=1.d0/(uk+um*s2)
               fx=1.d0+uk-uk2*fxden
               xchge(is)=ax*fx*roeth
c
               dfs=2.d0*uk2*fxden*fxden*um*s
               dfxd(is)=(ax*roeth*x43)*(fx-dfs*s)
               dfxdg(is)=ax*roeth*dfs*sd
            endif
         enddo
c     _________________________________________________________________
c     back to total charge density  
c        Be careful! Total charge density is not spin-scaled!
c
         if(isp) then
            roe=rhoe(ir,1)+rhoe(ir,2)
            if(roe.eq.0.d0) then
               rhoe(ir,:)=0.d0
               gradr(ir,:,:)=0.d0
               goto 100
            endif
            aroe(3)=abs(roe)
            grx=gradr(ir,1,3)
            gry=gradr(ir,2,3)
            grz=gradr(ir,3,3)
            agr(3)=sqrt(grx*grx+gry*gry+grz*grz)
            roeth=aroe(3)**x13
            rs= pider1/roeth
            gkf=pider2*roeth
            sd=1.d0/(2.0d0*gkf*aroe(3))
            s=agr(3)*sd
            s2=s*s
          endif
c     _________________________________________________________________
c     correlation ecr=ec(rho,zeta) 
c
         rs12=sqrt(rs)
         rs32=rs12*rs
         rs2=rs*rs
         rs3=rs*rs2
         ecden(1)=a*(bt1*rs12+bt2*rs+bt3*rs32+bt4*rs2) 
         eclog(1)=log(1.d0+(1.d0/ecden(1)))
         ecr(1)=-a*(1.d0+alfa1*rs)*eclog(1)
         if(isp) then
            ecden(2)=a_p*(bt1_p*rs12+bt2_p*rs+bt3_p*rs32+bt4_p*rs2) 
            eclog(2)=log(1.d0+(1.d0/ecden(2)))
            ecr(2)=-a_p*(1.d0+alfa1_p*rs)*eclog(2)
c
            alfden=a_alf*(bt1_alf*rs12+bt2_alf*rs
     $             +bt3_alf*rs32+bt4_alf*rs2) 
            alflog=log(1.d0+(1.d0/alfden))
cyy note the sign change for alfr
            alfr=a_alf*(1.d0+alfa1_alf*rs)*alflog
c
            zeta=(rhoe(ir,1)-rhoe(ir,2))/roe
            if(abs(zeta).gt.1.d0) zeta=dble(sign(1.d0,zeta))
            zeta3=zeta*zeta*zeta
            zeta4=zeta3*zeta
c           invgam=1.0/(2.d0**(3.d0/4.d0)-2.d0)
            fzeta=((1.d0+zeta)**x43+(1.d0-zeta)**x43-2.d0)*invgam
            fzetap=x43*((1.d0+zeta)**x13-(1.d0-zeta)**x13)*invgam
            fzetazeta4=fzeta*zeta4
            phi=((1.d0+zeta)**x23+(1.d0-zeta)**x23)*0.5d0
            phip=(((1.d0+zeta)**2+eta)**x16m 
     $           -((1.d0-zeta)**2+eta)**x16m)*x13
            invphi=1.d0/phi
            phi3=phi*phi*phi
            invphi3=1.d0/phi3
c
            ecr_dif=ecr(2)-ecr(1)
            ecr(3)=ecr(1)+alfr*fzeta*invfzz*(1.d0-zeta4)
     $          +ecr_dif*fzetazeta4
            decdz=4.d0*zeta3*fzeta*(ecr_dif-alfr*invfzz)
     $          +fzetap*(zeta4*ecr_dif+(1.d0-zeta4)*alfr*invfzz)
         endif
c     _________________________________________________________________
c     correlation h(t,ecr,zeta)
c
         td=pider3*sd/rs12*invphi
         t=agr(2*nspin-1)*td
         t2=t*t
         t3=t*t2
         t4=t2*t2
         aexp=exp(-abder2*ecr(2*nspin-1)*invphi3)-1.d0
         abig=abder3/aexp
         abig2=abig*abig
         hnum=t2+abig*t4
         hden=1.d0/(1.d0+abig*t2+abig2*t4)
         harg=1.d0+abder3*hnum*hden
         h=abder1*log(harg)*phi3
c     _________________________________________________________________
c     updating of xc-energy
c        Be cafeful! Exchange energy has 1/2 factor in front;
c        however, exchange potential do not have 1/2 factor if we
c        just use the form of unpolarized form with twice the n(+,-).
c
         exc=exc+(ecr(2*nspin-1)+h)*aroe(2*nspin-1)
         do is=1,nspin
            exc=exc+xchge(is)*aroe(is)*invspscale
         enddo    
c     _________________________________________________________________
c     first part xc-potential from ecr 
c
         decdrs(1)=-a*alfa1*eclog(1)*rs+a*(1.d0+alfa1*rs)
     $        *a*(0.5d0*bt1*rs12+bt2*rs+1.5d0*bt3*rs32+2.0d0*bt4*rs2)
     $        /(ecden(1)*ecden(1)+ecden(1))
         if(isp) then
            decdrs(2)=-a_p*alfa1_p*eclog(2)*rs+a_p*(1.d0+alfa1_p*rs)
     $          *a_p*(0.5d0*bt1_p*rs12+bt2_p*rs+1.5d0*bt3_p*rs32
     $          +2.0d0*bt4_p*rs2)/(ecden(2)*ecden(2)+ecden(2))
            dalfdrs=a_alf*alfa1_alf*alflog*rs-a_alf*(1.d0
     $          +alfa1_alf*rs)*a_alf*(0.5d0*bt1_alf*rs12+bt2_alf*rs
     $          +1.5d0*bt3_alf*rs32+2.0d0*bt4_alf*rs2) 
     $          /(alfden*alfden+alfden)
            decdrs(3)=decdrs(1)*(1-fzetazeta4)+decdrs(2)*fzetazeta4
     $          +dalfdrs*fzeta*invfzz*(1-zeta4)
         endif
         decd=-x13*decdrs(2*nspin-1)
c     _________________________________________________________________
c     first part xc-potential from h 
c
         dhda=phi3*abder1/harg*abder3*hden*(t4-hnum*hden*(t2+2.0d0
     $        *abig*t4))
         dadec=abder3*abder2*(aexp+1.d0)/(aexp*aexp)*invphi3
         if(isp) dadz=dadec*(decdz-ecr(2*nspin-1)*3.d0*invphi*phip)
         dhd=dhda*dadec*decd 
         dhdt=phi3*abder1/harg*abder3*hden
     $        *(2.0d0*t+4.0d0*abig*t3-hnum*hden*(2.0d0*abig*t+4.0d0
     $        *abig2*t3))
         dhd=dhd-x76*t*dhdt
         dhdg=dhdt*td
         if(isp) dhdz=dhda*dadz+(3.d0*h-dhdt*t)*phip*invphi
c     _________________________________________________________________
c     first part xc-potential 
c        Note the following for the spin-polarized calculation.
c        1. dexcd include dfxd: Exchange potential from the derivative
c           w.r.t spin-scale*n(+,-) is included in the xc-potential.
c           This is stored at rhoe(is) temporarily.
c        2. dexcdg ( defined by d exc d |grad n|) at this point do not
c           include the contribution from exchange: The dependence is 
c           through |grad n(+,-)| for exchange whereas the dependence
c           is through |grad n_tot| with zeta for correlation of PBE. 
c        3. Therefore, contribution from the exchange is prepared at
c           this stage and not included. Compare with unpolarized case. 
c
      do is=1,nspin
         dexcd(is)=dfxd(is)+decd+dhd+ecr(2*nspin-1)+h
      enddo
c        if agr is less than delt byagr is set to zero.
         isign=sign(1.d0,agr(2*nspin-1)-delt)
         byagr=0.5d0*(1+isign)/(agr(2*nspin-1)+(1-isign)*delt)
c
      if(isp) then
         dexcdg=dhdg*aroe(2*nspin-1)*byagr
         do is=1,nspin
            rhoe(ir,is)=dexcd(is) 
     $                -(zeta-2.d0*(1.5d0-is))*(decdz+dhdz)
            isign=sign(1.d0,agr(is)-delt)
            byagr=0.5d0*(1+isign)/(agr(is)+(1-isign)*delt)
            gradr(ir,:,is)=spscale*gradr(ir,:,is)*dfxdg(is)
     $                    *aroe(is)*byagr
         enddo
      else
         rhoe(ir,1)=dexcd(1)
         dexcdg=(dfxdg(1)+dhdg)*aroe(2*nspin-1)*byagr
      endif
c
         gradr(ir,:,2*nspin-1)=gradr(ir,:,2*nspin-1)*dexcdg
 100  continue
c     _________________________________________________________________
c     sum exc over all processors
      call all_sum_all(exc)
c     _________________________________________________________________
c     au to ry
c
      exc=exc*2.d0*normfac
c     _________________________________________________________________
c     second part xc-potential: 3 forward ffts  
c
c     gradr(:,k) now contains df_xc/d|grad n(r')| grad n(r')/|grad n(r')| 
c     for unpolarized case. For polarized case exchange is separate.
c
      do is=1,nspin
         gswork2(:)=0.d0
         do k=1,3
            if(isp) then
               rswork(:)=gradr(:,k,3)+gradr(:,k,is)
            else
               rswork(:)=gradr(:,k,1)
            endif
            call fourier_transform(1,ffts,gs,gswork(1),rswork(1))
            call derivative_gspace(k,gswork(1),gs,crys,gswork1(1))
            gswork2(:)=gswork2(:)+gswork1(:) * (0.d0,1.d0)
         enddo
c
c     _________________________________________________________________
c     second part xc-potential: 1 inverse fft  
c
         call fourier_transform(-1,ffts,gs,gswork2(1),rswork(1))
         rswork(:)=rhoe(:,is)-rswork(:)
c
c     _________________________________________________________________
c
         call fourier_transform(1,ffts,gs,inputcd(1,is),rswork(1))
      enddo
c
c     _________________________________________________________________
c     au to ry
c
      inputcd(:,:)=inputcd(:,:)*2.d0
C     _________________________________________________________________
      deallocate(rswork)
      deallocate(gswork)
      deallocate(gswork1)
      deallocate(gswork2)
c
      deallocate(rhoe)
      deallocate(gradr)
C     _________________________________________________________________
      return
      end
